################################################################################
import json, logging, socket, WagonTestGUI
from WagonTestGUI.PythonFiles.Data.DBSender import DBSender

FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'
logging.basicConfig(filename="{}/PythonFiles/logs/GUIWindow.log".format(WagonTestGUI.__path__[0]), filemode = 'w', format=FORMAT, level=logging.DEBUG)

class DataHolder():

    #################################################

    # List of the variables being held by data holder
    def __init__(self):
        
        # Object that sends information to the database
        self.data_sender = DBSender()
        
        self.data_dict = {
                'user_ID': "_",
                'test_stand': str(socket.gethostname()),
                'current_serial_ID': "-1BAD",
                'test1_completed': False,
                'test2_completed': False,
                'test3_completed': False,
                'test4_completed': False,
                'test1_pass': False,
                'test2_pass': False,
                'test3_pass': False,
                'test4_pass': False,
                'comments': "_",
                'is_new_board': False,
                'tests_run': [str(1), str(2), str(3), str(4)],
                }
        self.inspection_data = {
                'board_chipped_bent': False,
                'wagon_connection_pin_bent': False,
                'engine_connection_pin_bent': False,
                'visual_scratches': False,
                'inspection_comments': "_"
                }
        self.data_lists = {
                'test_results': [self.data_dict['test1_pass'], self.data_dict['test2_pass'], self.data_dict['test3_pass'], self.data_dict['test4_pass']],
                'test_completion': [self.data_dict['test1_completed'], self.data_dict['test2_completed'], self.data_dict['test3_completed'], self.data_dict['test4_completed']] 
                }
    
    #################################################

    def add_new_user_name(self, user_ID, passwd):
        self.data_dict['user_ID'] = user_ID
        
        is_new_user_ID = True

        for item in self.get_all_users():
            if self.data_dict['user_ID'] == item:
                is_new_user_ID = False
        print("\n\n\n\n\n\nIs the user new?:{}\n\n\n\n\n\n".format(is_new_user_ID))

        if is_new_user_ID:
            self.data_sender.add_new_user_ID(self.data_dict['user_ID'], passwd)        


    def check_if_new_board(self):
        # Send request for query
        print("testing if new board")
        self.data_dict['is_new_board'] = self.test_new_board(self.get_serial_ID())        
        print("result received:", self.data_dict['is_new_board'])
        if self.data_dict['is_new_board'] == False:
            prev_results = self.data_sender.get_previous_test_results(self.get_serial_ID())
            for result in prev_results:
                test_id = result[0]
                pass_fail = result[1]
                if pass_fail == 0:
                    pass_fail = False
                elif pass_fail == 1:
                    pass_fail = True
                for index, test in enumerate(self.data_dict['tests_run']):
                    if test_id == test:
                        if index == 0:
                            self.data_dict['test1_pass'] = pass_fail
                            self.data_dict['test1_completed'] = pass_fail
                        elif index == 1:
                            self.data_dict['test2_pass'] = pass_fail
                            self.data_dict['test2_completed'] = pass_fail
                        elif index == 2:
                            self.data_dict['test3_pass'] = pass_fail
                            self.data_dict['test3_completed'] = pass_fail
                        elif index == 3:
                            self.data_dict['test4_pass'] = pass_fail
                            self.data_dict['test4_completed'] = pass_fail
                        
        else:
            pass

    #################################################


    def set_user_ID(self, user_ID):

        print("\n\n\n\n\nuser_ID", user_ID)
 
        self.data_dict['user_ID'] = user_ID 
        logging.debug("DataHolder: User ID has been set.")

    ##################################################

    def set_serial_ID(self, sn):
        self.data_sender.add_new_board(sn)
                    
        self.data_dict['current_serial_ID'] = sn
        logging.info("DataHolder: Serial Number has been set.")

    

    def test_new_board(self, sn):
        logging.info("DataHolder: Checking if serial is a new board")
        return self.data_sender.is_new_board(sn)



    ##################################################

    def get_serial_ID(self):
        # TODO
        # self.data_sender.add_new_board(sn)

        return self.data_dict['current_serial_ID']

    #################################################

    # Future method to send data to the database
    def send_all_to_DB(self):
          
        person_ID = self.data_dict['user_ID']
        comments = self.data_dict['comments']
        serial_number = self.get_serial_ID()
        
         
        for i in range(len(self.data_dict['tests_run'])):
            print("Iteration:", i)
            temp = 0
            if self.data_list['test_results'][i]:
                temp = 1
            info_dict = {"serial_num":serial_number,"tester": person_ID, "test_type": self.tests_run[i], "successful": temp, "comments": comments} 
            with open("{}/PythonFiles/JSONFiles/storage.json".format(WagonTestGUI.__path__[0]), "w") as outfile:
                print(info_dict)
                json.dump(info_dict, outfile)
            self.data_sender.add_test_json("{}/PythonFiles/JSONFiles/storage.json".format(WagonTestGUI.__path__[0]))
        logging.info("DataHolder: All results sent to database.")
    #################################################

    def send_to_DB(self, test_run):
        index = test_run - 1
        
        file_path_list = [
            "{}/PythonFiles/JSONFiles/Current_GenRes_JSON.json".format(WagonTestGUI.__path__[0]),
            "{}/PythonFiles/JSONFiles/Current_IDRes_JSON.json".format(WagonTestGUI.__path__[0]),
            "{}/PythonFiles/JSONFiles/Current_IIC_JSON.json".format(WagonTestGUI.__path__[0]),
            "{}/PythonFiles/JSONFiles/Current_BERT_JSON.json".format(WagonTestGUI.__path__[0])
        ]
            

        # Converts self.test_results[index] into 1/0 instead of bool       
        temp = 0
        if self.data_lists['test_results'][index]:
            temp = 1 


        info_dict = {"serial_num":self.get_serial_ID(),"tester": self.data_dict['user_ID'], "test_type": self.data_dict['tests_run'][index], "successful": temp, "comments": self.data_dict['comments']}
        
        with open("{}/PythonFiles/JSONFiles/storage.json".format(WagonTestGUI.__path__[0]), "w") as outfile:
            print(info_dict)
            json.dump(info_dict, outfile)
        self.data_sender.add_test_json("{}/PythonFiles/JSONFiles/storage.json".format(WagonTestGUI.__path__[0]), file_path_list[index])
        logging.info("DataHolder: Test results sent to database.")

    #################################################
   
    def get_all_users(self):
        users_list = self.data_sender.get_usernames() 
        print ("\n users_list:", users_list)
        return users_list

    #################################################

    # Prints all the variable values inside data_holder
    def print(self):    
        print("data_dict: \n", self.data_dict, "\ninspection_data: \n", self.inspection_data)
    
    #################################################
    
    def update_from_json_string(self, imported_json_string):
        json_dict = json.loads(imported_json_string)
 
        test_type = json_dict["name"]

        if test_type == "General Resistance Test":
            with open("{}/PythonFiles/JSONFiles/Current_GenRes_JSON.json".format(WagonTestGUI.__path__[0]), "w") as file:
                json.dump(json_dict['data'], file)
            print("\n\n\n\n GENRES TEST \n\n\n\n")
            self.data_dict['user_ID'] = json_dict["tester"]
            self.data_dict['current_serial_ID'] = json_dict["board_sn"] 
            self.data_dict['test1_completed'] = True
            self.data_dict['test1_pass'] = json_dict["pass"]
            self.send_to_DB(1)


        elif test_type == "ID Resistance Test":
            with open("{}/PythonFiles/JSONFiles/Current_IDRes_JSON.json".format(WagonTestGUI.__path__[0]), "w") as file:
                json.dump(json_dict['data'], file)

            self.data_dict['user_ID'] = json_dict["tester"]
            self.data_dict['current_serial_ID'] = json_dict["board_sn"] 
            self.data_dict['test2_completed'] = True
            self.data_dict['test2_pass'] = json_dict["pass"]
            self.send_to_DB(2)


        elif test_type == "IIC Check":
            with open("{}/PythonFiles/JSONFiles/Current_IIC_JSON.json".format(WagonTestGUI.__path__[0]), "w") as file:
                json.dump(json_dict['data'], file)

            self.data_dict['user_ID'] = json_dict["tester"]
            self.data_dict['current_serial_ID'] = json_dict["board_sn"] 
            self.data_dict['test3_completed'] = True
            self.data_dict['test3_pass'] = json_dict["pass"]
            self.send_to_DB(3)

        elif test_type == "Bit Error Rate Test":
            with open("{}/PythonFiles/JSONFiles/Current_BERT_JSON.json".format(WagonTestGUI.__path__[0]), "w") as file:
                json.dump(json_dict['data'], file)
            self.data_dict['user_ID'] = json_dict["tester"]
            self.data_dict['current_serial_ID'] = json_dict["board_sn"] 
            self.data_dict['test4_completed'] = True
            self.data_dict['test4_pass'] = json_dict["pass"]
            self.send_to_DB(4)

        # Updates the lists
        self.data_lists['test_results'] = [self.data_dict['test1_pass'], self.data_dict['test2_pass'], self.data_dict['test3_pass'], self.data_dict['test4_pass']]
        self.data_lists['test_completion'] = [self.data_dict['test1_completed'], self.data_dict['test2_completed'], self.data_dict['test3_completed'], self.data_dict['test4_completed']]

        logging.info("DataHolder: Test results have been saved")

    ################################################

    def add_inspection_to_comments(self):
        if self.inspection_data['board_chipped_bent']:
            if self.data_dict['comments'] == "_":
                self.data_dict['comments'] = ""
            self.data_dict['comments'] = self.data_dict['comments'] + " Board is chipped or bent."
        if self.inspection_data['wagon_connection_pin_bent']:
            if self.data_dict['comments'] == "_":
                self.data_dict['comments'] = ""
            self.data_dict['comments'] = self.data_dict['comments'] + " Wagon connnection pin is bent."
        if self.inspection_data['engine_connection_pin_bent']:
            if self.data_dict['comments'] == "_":
                self.data_dict['comments'] = ""
            self.data_dict['comments'] = self.data_dict['comments'] + " Engine connection pin is bent."
        if self.inspection_data['visual_scratches']: 
            if self.data_dict['comments'] == "_":
                self.data_dict['comments'] = ""
            self.data_dict['comments'] = self.data_dict['comments'] + " There are visual scratches on the board."
        if self.inspection_data['inspection_comments'] != "_": 
            if self.data_dict['comments'] == "_":
                self.data_dict['comments'] = ""
            self.data_dict['comments'] = self.data_dict['comments'] + " User comments: " + self.inspection_data['inspection_comments']
   
    ################################################

    # Keeps the login information stored
    def data_holder_new_test(self): 
        self.data_dict['current_serial_ID'] = "-1BAD"  
        self.data_dict['test1_completed'] = False
        self.data_dict['test2_completed'] = False
        self.data_dict['test3_completed'] = False
        self.data_dict['test4_completed'] = False
        self.data_dict['test1_pass'] = False     
        self.data_dict['test2_pass'] = False     
        self.data_dict['test3_pass'] = False     
        self.data_dict['test4_pass'] = False 
        self.data_dict['comments'] = "_"
        self.data_dict['is_new_board'] = False

        self.data_lists['test_results'] = [self.data_dict['test1_pass'], self.data_dict['test2_pass'], self.data_dict['test3_pass'], self.data_dict['test4_pass']]
        self.data_lists['test_completion'] = [self.data_dict['test1_completed'], self.data_dict['test2_completed'], self.data_dict['test3_completed'], self.data_dict['test4_completed']]

        self.inspection_data['board_chipped_bent'] = False
        self.inspection_data['wagon_connection_pin_bent'] = False
        self.inspection_data['engine_connection_pin_bent'] = False
        self.inspection_data['visual_scratches'] = False
 
        logging.info("DataHolder: DataHolder Information has been reset for a new test.")        

    ################################################

#################################################################################



# Importing necessary modules
from audioop import mul
import multiprocessing as mp
import socket
# Imports the GUIWindow
from PythonFiles.StressTest import StressTest
from PythonFiles.utils.SUBClient import SUBClient


#####################################################################
#                                                                   #
#            Please go to PythonFiles/StressTest.py                 #
#          to set up the settings for the Stress Test.              #
#         This file is essentially an executable for the            #
#                       stress test script.                         # 
#                                                                   #
#####################################################################


# Creates a task of creating the GUIWindow
def task_GUI(conn, queue):
    # creates the main_window as an instantiation of GUIWindow
    stress_test = StressTest(conn, queue)

# Creates a task of creating the SUBClient
def task_SUBClient(conn, queue):
    # Creates the SUBSCRIBE Socket Client
    sub_client = SUBClient(conn, queue)

def run():    
    # Creates a Pipe for the SUBClient to talk to the GUI Window
    conn_SUB, conn_GUI = mp.Pipe()

    queue = mp.Queue()

    # Turns creating the GUI and creating the SUBClient tasks into processes
    process_test = mp.Process(target = task_GUI, args=(conn_GUI, queue,))
    process_SUBClient = mp.Process(target = task_SUBClient, args = (conn_SUB, queue,))
    

    # Starts the processes
    process_test.start()
    process_SUBClient.start()

    # Should hold the code at this line until the GUI process ends
    process_test.join()

    try:
        conn_SUB.close()
        conn_GUI.close()
    except:
        print("Pipe close is unnecessary.")

    try:
        # Cleans up the SUBClient process
        process_SUBClient.terminate()
    except:
        print("Terminate is unnecessary.")
        pass

if __name__ == "__main__":
    run()



import requests
import json
import WagonTestGUI
# from read_barcode import read_barcode


class DBSender():

    # Empty constructor
    def __init__(self) :
        pass


    def add_new_user_ID(self, user_ID, passwd):
        r = requests.post('http://cmslab3.umncmslab/~cros0400/cgi-bin/add_tester2.py', data= {'person_name':user_ID, 'password': passwd})



    # Returns an acceptable list of usernames from the database
    def get_usernames(self):
        r = requests.get('http://cmslab3.umncmslab/~cros0400/cgi-bin/get_usernames.py')
        lines = r.text.split('\n')


        begin = lines.index("Begin") + 1
        end = lines.index("End")

        usernames = []

        for i in range(begin, end):
            temp = lines[i]
            usernames.append(temp)

        return usernames

        

    # Returns a list of booleans
    # Whether test (by index) has been completed or not
    def get_test_completion_staus(self, serial_number):
        r = requests.post('http://cmslab3.umncmslab/~cros0400/cgi-bin/get_test_completion_status.py', data= serial_number)
        
        lines = r.text.split('\n')
        begin = lines.index("Begin") + 1 
        end = lines.index("End")
        
        tests_completed = []
        for i in range(begin, end):
            temp = lines[i][1:-1].split(",")
            temp[0] = str(temp[0])
            temp[1] = int(temp[1])
            tests_completed.append(temp)

        return tests_completed



    # Returns a list of booleans
    # Whether or not DB has passing results 
    def get_previous_test_results(self, serial_number):
        r = requests.post('http://cmslab3.umncmslab/~cros0400/cgi-bin/get_previous_test_results.py', data={'serial_number': str(serial_number)})
        
        lines = r.text.split('\n')

        begin = lines.index("Begin") + 1
        end = lines.index("End")

        tests_passed= []


        for i in range(begin, end):
            temp = lines[i][1:-1].split(",")
            temp[0] = str(temp[0])
            temp[1] = int(temp[1])
            tests_passed.append(temp)

        return tests_passed

    
    
    # #TODO Verify if a board has already been instantiated with SN
    # Posts a new board with passed in serial number
    def add_new_board(self, sn):
        r = requests.post('http://cmslab3.umncmslab/~cros0400/cgi-bin/add_module2.py', data={"serial_number": str(sn)})





    def is_new_board(self, sn):
        
        r = requests.post('http://cmslab3.umncmslab/~cros0400/cgi-bin/is_new_board.py', data={"serial_number": str(sn)})
        
        lines = r.text.split('\n')
       
        begin = lines.index("Begin") + 1
        end = lines.index("End")

    
        for i in range(begin, end): 
            
            if lines[i] == "True":
                return True
            elif lines[i] == "False":
                return False






    # Posts information via the "info" dictionary
    # Serial number is within the info dictionary
    def add_board_info(self, info):
        r = requests.post('http://cmslab3.umncmslab/~cros0400/cgi-bin/add_board_info2.py', data = info)
    
    def add_initial_tests(self, results):
        r = requests.post('http://cmslab3.umncmslab/~cros0400/cgi-bin/add_init_test.py', data = results)
        
    def add_general_test(self, results, files):
        r = requests.post('http://cmslab3.umncmslab/~cros0400/cgi-bin/add_test2.py', data = results, files=files)

    def add_test_json(self, json_file, datafile_name):
        load_file = open(json_file)
        results = json.load(load_file)        
        load_file.close()

        datafile = open(datafile_name, "rb")        

        attach_data = {'attach1': datafile}
        print("Read from json file:", results)
        r = requests.post('http://cmslab3.umncmslab/~cros0400/cgi-bin/add_test_json.py', data = results, files = attach_data)


 # Returns a list of all different types of tests
    def get_test_list(self):
        r = requests.get('http://cmslab3.umncmslab/~cros0400/cgi-bin/get_test_types.py')

        lines = r.text.split('\n')

        begin = lines.index("Begin") + 1
        end = lines.index("End")

        tests = []

        for i in range(begin, end):
            temp = lines[i][1:-1].split(",")
            temp[0] = str(temp[0][1:-1])
            temp[1] = int(temp[1])
            tests.append(temp)

        return tests




################################################################################

# Importing necessary modules
import tkinter as tk
from PIL import ImageTk as iTK
from PIL import Image
import logging
import WagonTestGUI

#################################################################################

FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'
logging.basicConfig(filename="{}/PythonFiles/logs/GUIWindow.log".format(WagonTestGUI.__path__[0]), filemode = 'w', format=FORMAT, level=logging.DEBUG)

class SplashScene(tk.Frame):

    #################################################

    def __init__(self, parent, master_frame):
        self.initialize_GUI(parent, master_frame)

    #################################################
    
    def initialize_GUI(self, parent, master_frame):
        super().__init__(master_frame, width = 850, height = 500)

        logging.info("SplashScene: Frame has been initialized.")
        # Creating Bethel Logo
        img_bethel_logo = Image.open("{}/PythonFiles/Images/Bethel_Logo.png".format(WagonTestGUI.__path__[0]))
        img_bethel_logo = img_bethel_logo.resize((250,100), Image.ANTIALIAS)
        phimg_bethel_logo = iTK.PhotoImage(img_bethel_logo)
        lbl_bethel_logo = tk.Label(self, image=phimg_bethel_logo, width=250, height=100)
        lbl_bethel_logo.image = phimg_bethel_logo

        lbl_bethel_logo.grid(row=0, column= 0, padx = 50, pady = 100)

        # Creating UMN Logo
        img_umn_logo = Image.open('{}/PythonFiles/Images/UMN_Logo.png'.format(WagonTestGUI.__path__[0]))
        img_umn_logo = img_umn_logo.resize((250,100), Image.ANTIALIAS)
        phimg_umn_logo = iTK.PhotoImage(img_umn_logo)
        lbl_umn_logo = tk.Label(self, image=phimg_umn_logo, width=250, height=100)
        lbl_umn_logo.image = phimg_umn_logo

        lbl_umn_logo.grid(row = 0 , column = 2, padx = 50, pady = 100)

        # Creating label for names
        lbl_names = tk.Label(
            self,
            text = ' Created by:\n \n Bryan Crosman, \n Andrew Kirzeder, \n & \n Garrett Schindler',
            font = ('Arial', 15)
        )
        lbl_names.grid(row = 1, column = 1)

        self.grid_propagate(0)

    #################################################


#################################################################################



#################################################################################

# Importing Necessary Modules
import tkinter as tk
import tkinter.font as font


#################################################################################


# Creating class for the window
class Inspection1(tk.Frame):

    #################################################

    def __init__(self, parent, master_frame, data_holder):
        super().__init__(master_frame, width=850, height=500)

        self.test_name = "SOMETHING STRING"
        self.data_holder = data_holder
        
        self.update_frame(parent)

    #################################################

    def update_frame(self, parent):

        # Creates a font to be more easily referenced later in the code
        font_scene = ('Arial', 12)
        font_scene_14 = ('Arial', 14)

        # Create a centralized window for information
        frm_window = tk.Frame(self, width = 850, height = 500)
        frm_window.grid(column=0, row=0)

        # Create a label for the tester's name
        lbl_tester = tk.Label(
            frm_window, 
            text = "Tester: ", 
            font = font_scene
            )
        lbl_tester.grid(row=0, column=0, pady=15 )

        # Create an entry for the tester's name
        ent_tester = tk.Entry(
            frm_window, 
            font = font_scene
            )
        ent_tester.insert(0, self.data_holder.data_dict['user_ID'])
        ent_tester.grid(row=0, column=1, pady=15 )
        ent_tester.config(state = "disabled")

        # Create a label for the serial number box
        lbl_snum = tk.Label(
            frm_window, 
            text = "Serial Number: ", 
            font = font_scene
            )
        lbl_snum.grid(row=0, column=2, pady=15 )

        # Create a entry for the serial number box
        ent_snum = tk.Entry(
            frm_window, 
            font = font_scene
            )
        ent_snum.insert(0, self.data_holder.data_dict['current_serial_ID'])
        ent_snum.grid(row=0, column=3, pady=15)
        ent_snum.config(state = "disabled")


        self.board_chipped_bent = tk.BooleanVar()
        self.wagon_connection_pin_bent = tk.BooleanVar()
        self.engine_connection_pin_bent = tk.BooleanVar()
        self.visual_scratches = tk.BooleanVar()
        


        # Beginning of the CheckButtons
        
        # Checkbutton1
        c1 = tk.Checkbutton(
            frm_window, 
            font = font_scene_14,
            text='Board Chipped/Bent',
            variable= self.board_chipped_bent, 
            onvalue=1, 
            offvalue=0 
            # command=print_selection
            )
        c1.grid(row = 1, column= 1, sticky='w', columnspan=2)
        
        # Checkbutton2
        c2 = tk.Checkbutton(
            frm_window, 
            font = font_scene_14,
            text='Wagon Connection Pin Bent',
            variable= self.wagon_connection_pin_bent, 
            onvalue=True, 
            offvalue=False 
            # command=print_selection
            )
        c2.grid(row = 2, column= 1, sticky='w', columnspan=2)

        # Checkbutton3
        c3 = tk.Checkbutton(
            frm_window, 
            font = font_scene_14,
            text='Engine Connection Pin Bent',
            variable= self.engine_connection_pin_bent, 
            onvalue=True, 
            offvalue=False 
            # command=print_selection
            )
        c3.grid(row = 3, column= 1, sticky='w', columnspan=2)

        # Checkbutton4
        c4 = tk.Checkbutton(
            frm_window, 
            font = font_scene_14,
            text='Visual Scratches',
            variable= self.visual_scratches, 
            onvalue=True, 
            offvalue=False 
            # command=print_selection
        )
        c4.grid(row = 4, column= 1, sticky='w', columnspan=2)




        

        # Create a label for the serial number box
        lbl_snum = tk.Label(
            frm_window, 
            text = "Comments:", 
            font = font_scene
            )
        lbl_snum.grid(row=5, column=1, pady=(25, 0) )

        # Comment Box
        self.comment_box = tk.Entry(
            frm_window,
            font = font_scene,
            state= 'normal',
            width= 75,
        )
        self.comment_box.grid(row = 6, column =1, sticky='w', columnspan=5)





    

        # Create a button for confirming test
        btn_confirm = tk.Button(
            frm_window, 
            text = "Confirm", 
            relief = tk.RAISED, 
            command = lambda:self.btn_confirm_action(parent)
            )
        btn_confirm.grid(row = 9, column= 1, pady= 50)
        btn_confirm['font'] = font.Font(family = 'Arial', size = 13)






        # Create frame for logout button
        nav_frame = tk.Frame(self)
        nav_frame.grid(column = 1, row = 0, sticky = 'ne', padx =5)


        # Create a rescan button
        btn_rescan = tk.Button(
            nav_frame, 
            text = "Change Boards", 
            relief = tk.RAISED, 
            command = lambda: self.btn_rescan_action(parent))
        btn_rescan.pack(anchor = 'ne', pady=15)

        # Create a logout button
        btn_logout = tk.Button(
            nav_frame, 
            text = "Logout", 
            relief = tk.RAISED, 
            command = lambda: self.btn_logout_action(parent))
        btn_logout.pack(anchor = 'se')

        # # # # # # # # # 

        
        self.grid_columnconfigure(4, weight=1)
        self.grid_rowconfigure(0, weight=1)

        
        frm_window.grid_propagate(0)
        self.grid_propagate(0)
        
    #################################################

    # Rescan button takes the user back to scanning in a new board
    def btn_rescan_action(self, _parent):
        _parent.set_frame_scan_frame()

    #################################################

    # Back button action takes the user back to the scanning device
    def btn_back_action(self, _parent):
        pass
    
    #################################################

    def update_data_holder(self):
        self.data_holder.inspection_data['board_chipped_bent'] = self.board_chipped_bent.get()
        self.data_holder.inspection_data['wagon_connection_pin_bent'] = self.wagon_connection_pin_bent.get()
        self.data_holder.inspection_data['engine_connection_pin_bent'] = self.engine_connection_pin_bent.get()
        self.data_holder.inspection_data['visual_scratches'] = self.visual_scratches.get()
        self.data_holder.inspection_data['inspection_comments'] = self.comment_box.get()
        self.data_holder.add_inspection_to_comments()
        self.data_holder.print()


    #################################################

    # Confirm button action takes the user to the test in progress scene
    def btn_confirm_action(self, _parent):
        
        self.update_data_holder()
        _parent.go_to_next_test()

        # # # # # # # # # # # # # # # # # # # # # # # # # # #
        #   ++ GOAL CODE ++                                 #
        # def confirm():                                    #
        #       set_frame_TIPS()                            #
        #       Runs_Test()   # Might include multithread   #
        #       Get_Results()                               #
        #       Update_Dataholder()                         #
        #       Go_To_Next_Test()                           #
        # # # # # # # # # # # # # # # # # # # # # # # # # # # 
        pass

    #################################################

    # functionality for the logout button
    def btn_logout_action(self, _parent):
        _parent.set_frame_login_frame()

    #################################################



#################################################################################

# importing necessary modules
import multiprocessing as mp
import logging, time
import tkinter as tk
import sys, time
from tkinter import *
from turtle import back
from PIL import ImageTk as iTK
from PIL import Image
import WagonTestGUI
 

#################################################################################

FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'
logging.basicConfig(filename="{}/PythonFiles/logs/GUIWindow.log".format(WagonTestGUI.__path__[0]), filemode = 'w', format=FORMAT, level=logging.DEBUG)


# creating the Scan Frame's class (called ScanScene) to be instantiated in the GUIWindow
# instantiated as scan_frame by GUIWindow
# @param parent -> passes in GUIWindow as the parent.
# @param master_frame -> passes master_frame as the container for everything in the class.
# @param data_holder -> passes data_holder into the class so the data_holder functions can
#       be accessed within the class.
class ScanScene(tk.Frame):
    
    #################################################

    # Runs upon creation
    def __init__(self, parent, master_frame, data_holder):
        self.data_holder = data_holder
        self.is_current_scene = False
        
        self.EXIT_CODE = 0
        # Runs the initilize_GUI function, which actually creates the frame
        # params are the same as defined above
        self.initialize_GUI(parent, master_frame)
        

    # Creates a thread for the scanning of a barcode
    # Needs to be updated to run the read_barcode function in the original GUI
    def scan_QR_code(self, master_window):
        
        self.ent_snum.config(state = 'normal')
        self.ent_snum.delete(0,END)
        self.master_window = master_window
        self.hide_rescan_button()

        sys.path.insert(1,'/home/hgcal/WagonTest/WagonTestGUI/PythonFiles/Scanner/python')

        from ..Scanner.python.get_barcodes import scan, listen, parse_xml

        manager = mp.Manager()
        serial = manager.list()

        self.ent_snum.config(state = 'normal')

        logging.info("ScanScene: Beginning scan...")
        self.scanner = scan()
        self.listener = mp.Process(target=listen, args=(serial, self.scanner))

        self.listener.start()
               
        while 1 > 0:

            try:
                self.master_window.update()
            except:
                pass
            if not len(serial) == 0:
                self.data_holder.set_serial_ID( parse_xml(serial[0]))

                self.listener.terminate()
                self.scanner.terminate()
               
                self.ent_snum.delete(0,END)
                self.ent_snum.insert(0, str(self.data_holder.get_serial_ID()))
                self.ent_snum.config(state = 'disabled')
                self.show_rescan_button()
                break

            elif self.EXIT_CODE:
                logging.info("ScanScene: Exit code received. Terminating processes.")
                self.listener.terminate()
                self.scanner.terminate()
                logging.info("ScanScene: Processes terminated successfully.")
                break
            else:
                time.sleep(.01)
            
        logging.info("ScanScene: Scan complete.")

    # Creates the GUI itself
    def initialize_GUI(self, parent, master_frame):
        
        self.master_frame = master_frame
        
        super().__init__(self.master_frame, width = 850, height = 500)

        logging.info("ScanScene: Frame has been created.")
        # Create a photoimage object of the QR Code
        QR_image = Image.open("{}/PythonFiles/Images/QRimage.png".format(WagonTestGUI.__path__[0]))
        QR_PhotoImage = iTK.PhotoImage(QR_image)
        QR_label = tk.Label(self, image=QR_PhotoImage)
        QR_label.image = QR_PhotoImage

        # the .grid() adds it to the Frame
        QR_label.grid(column=1, row = 0)

        Scan_Board_Prompt_Frame = Frame(self,)
        Scan_Board_Prompt_Frame.grid(column=0, row = 0)

        # creates a Label Variable, different customization options
        lbl_scan = tk.Label(
            master= Scan_Board_Prompt_Frame,
            text = "Scan the QR Code on the Board",
            font = ('Arial', 18)
        )
        lbl_scan.pack(padx = 50, pady = 50)

        # Create a label to label the entry box
        lbl_snum = tk.Label(
            Scan_Board_Prompt_Frame,
            text = "Serial Number:",
            font = ('Arial', 16)
        )
        lbl_snum.pack(padx = 20)

        # Entry for the serial number to be displayed. Upon Scan, update and disable?
        global ent_snum
        
        # Creating intial value in entry box
        user_text = tk.StringVar(self)
        
        # Creates an entry box
        self.ent_snum = tk.Entry(
            Scan_Board_Prompt_Frame,
            font = ('Arial', 16),
            textvariable= user_text, 
            )
        self.ent_snum.pack(padx = 50)

        # Traces an input to show the submit button once text is inside the entry box
        user_text.trace(
            "w", 
            lambda name, 
            index, 
            mode, 
            sv=user_text: self.show_submit_button()
            )

        # Rescan button creation
        self.btn_rescan = tk.Button(
            Scan_Board_Prompt_Frame,
            text="Rescan",
            padx = 20,
            pady =10,
            relief = tk.RAISED,
            command = lambda:  self.scan_QR_code(self.master_window)
            )
        self.btn_rescan.pack(pady=30)

        # Submit button creation
        self.btn_submit = tk.Button(
            Scan_Board_Prompt_Frame,
            text="Submit",
            padx = 20,
            pady = 10,
            relief = tk.RAISED,
            command= lambda:  self.btn_submit_action(parent)
            )
        self.btn_submit.pack()

        # Creating frame for logout button
        frm_logout = tk.Frame(self)
        frm_logout.grid(column = 1, row = 1, sticky= 'se')

        # Creating the logout button
        btn_logout = tk.Button(
            frm_logout,
            relief = tk.RAISED,
            text = "Logout",
            command = lambda: self.btn_logout_action(parent)
        )
        btn_logout.pack(anchor = 'se', padx = 0, pady = 80)
        # Locks frame size to the master_frame size
        self.grid_propagate(0)

    #################################################

    # Function for the submit button
    def btn_submit_action(self, _parent):
        self.data_holder.set_serial_ID(self.ent_snum.get())
        self.data_holder.check_if_new_board()
        _parent.scan_frame_progress()


    #################################################

    # Function for the log out button
    def btn_logout_action(self, _parent):

         # Send user back to login frame
        _parent.set_frame_login_frame() 

    #################################################

    # Function to activate the submit button
    def show_submit_button(self):
        self.btn_submit["state"] = "active"

    #################################################

    # Function to disable to the submit button
    def hide_submit_button(self):
        self.btn_submit["state"] = "disabled"

    #################################################

    # Function to activate the rescan button
    def show_rescan_button(self):
        self.btn_rescan["state"] = "active"

    #################################################

    # Function to disable to the rescan button
    def hide_rescan_button(self):
        self.btn_rescan["state"] = "disabled"

    #################################################
        
    def kill_processes(self):
        logging.info("ScanScene: Terminating scanner proceses.")
        try:
            self.scanner.kill()
            self.listener.terminate()
            self.EXIT_CODE = 1
        except:
            logging.info("ScanScene: Processes could not be terminated.")



#################################################################################

# Imports all the necessary modules
import tkinter as tk
from tkinter import ttk
from xml.dom.expatbuilder import parseFragmentString
import time
import logging
import WagonTestGUI

#################################################################################

FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'
logging.basicConfig(filename="{}/PythonFiles/logs/GUIWindow.log".format(WagonTestGUI.__path__[0]), filemode = 'w', format=FORMAT, level=logging.DEBUG)

# Creating the frame itself
class TestInProgressScene(tk.Frame):
    def __init__(self, parent, master_frame, data_holder, queue, _conn):
        super().__init__(master_frame, width = 850, height = 500)


        self.queue = queue
        self.data_holder = data_holder
        self.is_current_scene = False
        self.initialize_scene(parent, master_frame)
        self.conn = _conn

    ##################################################

    # A function for the stop button
    def btn_stop_action(self, _parent):

        _parent.go_to_next_test()

        
        # Destroys the console window
        self.console_destroy()
        
    #################################################    

    # Goes to the next scene after the progress scene is complete
    def go_to_next_frame(self, _parent):
        _parent.go_to_next_test()

    #################################################    

    # Used to bring the user back to the test that just failed
    def go_to_previous_frame(self, _parent, previous_frame):
        self.previous_frame = previous_frame
        _parent.set_frame(previous_frame)


    # Used to initialize the frame that is on the main window
    # next_frame is used to progress to the next scene and is passed in from GUIWindow
    def initialize_scene(self, parent, master_frame):
        
        logging.info("TestInProgressScene: The frame has been initialized.")
        scrollbar = tk.Scrollbar(self)
        scrollbar.pack(side = "right", fill = 'y')


        # Placing an entry box in the frm_console
        global ent_console
        ent_console = tk.Text(
            self, 
            bg = 'black', 
            fg = 'white', 
            height= 15,
            width= 400,
            font = ('Arial', 15),
            yscrollcommand = scrollbar.set
            )
        

        # Adding scrollbar functionality
        scrollbar.config(command = ent_console.yview)


        # Creating the main title in the frame
        lbl_title = tk.Label(self, 
            text = "Test in progress. Please wait.", 
            font = ('Arial', 20)
            )
        lbl_title.pack(padx = 0, pady = 50)

        # Create a progress bar that does not track progress but adds motion to the window
        self.prgbar_progress = ttk.Progressbar(
            self, 
            orient = 'horizontal',
            mode = 'indeterminate', length = 350)
        self.prgbar_progress.pack(padx = 50)
        self.prgbar_progress.start()

        print("\n\n\n\n\n\n\n\n Starting Progress Bar \n\n\n\n\n\n\n")

        # A Button To Stop the Progress Bar and Progress Forward (Temporary until we link to actual progress)
        btn_stop = ttk.Button(
            self, 
            text='Stop', 
            command= lambda: self.btn_stop_action(parent))
        btn_stop.pack(padx = 0, pady = 25)

        ent_console.pack(anchor = 'center')



        # Forces the frame to stay the size of the master_frame
        self.pack_propagate(0)

    # A function for the stop button
    def btn_stop_action(self, _parent):

        _parent.go_to_next_test()



    # Goes to the next scene after the progress scene is complete
    def go_to_next_frame(self, _parent):
        _parent.go_to_next_test()

        

    # Used to bring the user back to the test that just failed
    def go_to_previous_frame(self, _parent, previous_frame):
        self.previous_frame = previous_frame
        _parent.set_frame(previous_frame)

    #################################################

    def begin_update(self, master_window, queue):
        logging.info("TestInProgressScene: Started console update loop.")
        # try:
        while 1>0:
                # try:
            master_window.update()
            if not queue.empty():    
                logging.info("TestInProgressScene: Waiting for queue objects...")
                text = queue.get()
                print(text)
                ent_console.insert(tk.END, text)
                ent_console.insert(tk.END, "\n")
                ent_console.see('end')

                if text == "Results received successfully.":
                
                    message =  self.conn.recv()
                    
                    print("\n\nmessage:",message , "\n\n")
                    self.data_holder.update_from_json_string(message) 
                    
                    logging.info("TestInProgressScene: JSON Received.")
                    master_window.update()
                    time.sleep(1)
                    break
                
            else:
                time.sleep(.01)

    def close_prgbar(self):
        logging.debug("TestInProgressScene: Closing the progressbar.")
        self.prgbar_progress.stop()
        self.prgbar_progress.destroy()
        logging.debug("TestInProgressScene: Progressbar succesfully closed.")



#!/usr/bin/python3

# Importing necessary modules
from audioop import mul
import multiprocessing as mp
import socket
# Imports the GUIWindow
from WagonTestGUI.PythonFiles.GUIWindow import GUIWindow
from WagonTestGUI.PythonFiles.utils.SUBClient import SUBClient
import os

# Creates a task of creating the GUIWindow
def task_GUI(conn, queue):
    # creates the main_window as an instantiation of GUIWindow
    main_window = GUIWindow(conn, queue)

# Creates a task of creating the SUBClient
def task_SUBClient(conn, queue):
    # Creates the SUBSCRIBE Socket Client
    sub_client = SUBClient(conn, queue)

def run():    
    # Creates a Pipe for the SUBClient to talk to the GUI Window
    conn_SUB, conn_GUI = mp.Pipe()

    queue = mp.Queue()

    # Turns creating the GUI and creating the SUBClient tasks into processes
    process_GUI = mp.Process(target = task_GUI, args=(conn_GUI, queue,))
    process_SUBClient = mp.Process(target = task_SUBClient, args = (conn_SUB, queue,))
    

    # Starts the processes
    process_GUI.start()
    process_SUBClient.start()

    # Should hold the code at this line until the GUI process ends
    process_GUI.join()

    try:
        conn_SUB.close()
        conn_GUI.close()
    except:
        print("Pipe close is unnecessary.")

    try:
        # Cleans up the SUBClient process
        process_SUBClient.terminate()
    except:
        print("Terminate is unnecessary.")
        pass

def main(args):
    pass

if __name__ == "__main__":

    curpath = os.path.abspath(os.curdir)
    print( "Current path is: %s" % (curpath))

    print(socket.gethostname())
    ###### Example code to branch between the different GUIS #####
    # visual_GUI_computers = [
    # computer_1,
    # computer_2,
    # etc.
    # ]
    # wagon_GUI_computers = [
    # computer_3,
    # computer_4,
    # etc.
    # ]
    # engine_GUI_computers = [
    # computer_5,
    # computer_6,
    # etc.
    # ]
    # current_computer = socket.gethostname()
    # for computer in visual_GUI_computers:
    #    if current_computer == computer:
    #        run_visual_GUI()
    #    else:
    #        pass
    # for computer in wagon_GUI_computers:
    #    if current_computer == computer:
    #        run_wagon_GUI()
    #    else:
    #        pass
    # for computer in engine_GUI_computers:
    #    if current_computer == computer:
    #        run_engine_GUI()
    #    else:
    #        pass
    ###### END EXAMPLE CODE ######
    run()



#################################################################################

# Importing all neccessary modules
from pickle import NONE
import tkinter as tk
from turtle import bgcolor
import multiprocessing as mp
import logging
#from pyparsing import trace_parse_action

# Importing all the neccessary files and classes from them
from WagonTestGUI.PythonFiles.Scenes.SidebarScene import SidebarScene
from WagonTestGUI.PythonFiles.Scenes.LoginScene import LoginScene
from WagonTestGUI.PythonFiles.Scenes.ScanScene import ScanScene
from WagonTestGUI.PythonFiles.TestFailedPopup import TestFailedPopup
from WagonTestGUI.PythonFiles.Scenes.TestSummaryScene import TestSummaryScene
from WagonTestGUI.PythonFiles.Scenes.TestScene import *
from WagonTestGUI.PythonFiles.Scenes.TestInProgressScene import TestInProgressScene
from WagonTestGUI.PythonFiles.Data.DataHolder import DataHolder
from WagonTestGUI.PythonFiles.Scenes.SplashScene import SplashScene
from WagonTestGUI.PythonFiles.Scenes.TestInProgressScene import *
from WagonTestGUI.PythonFiles.Scenes.Inspection1 import Inspection1
from WagonTestGUI.PythonFiles.Scenes.AddUserScene import AddUserScene

#################################################################################



FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'
logging.basicConfig(filename="{}/PythonFiles/logs/GUIWindow.log".format(WagonTestGUI.__path__[0]), filemode = 'w', format=FORMAT, level=logging.DEBUG)

# Create a class for creating the basic GUI Window to be called by the main function to
# instantiate the actual object
class GUIWindow():

    #################################################

    def __init__(self, conn, queue):
        self.conn = conn
        self.queue = queue
        self.retry_attempt = False
        self.completed_window_alive = False
        self.current_test_index = 0
                             
        # Create the window named "self.master_window"
        # global makes self.master_window global and therefore accessible outside the function
        self.master_window = tk.Tk()
        self.master_window.title("Bethel Interns' Window")
        # Creates the size of the window and disables resizing
        self.master_window.geometry("1063x500+25+100")
        self.master_window.resizable(0,0)

        # Removes the tkinter logo from the window
        # self.master_window.wm_attributes('-toolwindow', 'True')

        # Creates and packs a frame that exists on top of the master_frame
        self.master_frame = tk.Frame(self.master_window, width=850, height= 500)
        self.master_frame.grid(column = 1, row = 0, columnspan = 4)

        # Creates a frame to house the sidebar on self.master_window
        sidebar_frame = tk.Frame(self.master_window, width = 213, height = 500)
        sidebar_frame.grid(column = 0 , row = 0)

        # Creates the "Storage System" for the data during testing
        self.data_holder = DataHolder()

        # Creates all the widgets on the sidebar
        self.sidebar = SidebarScene(self, sidebar_frame, self.data_holder)
        self.sidebar.pack()

        #################################################
        #   Creates all the different frames in layers  #
        #################################################

        # At top so it can be referenced by other frames' code... Order of creation matters

        self.test_summary_frame = TestSummaryScene(self, self.master_frame, self.data_holder)
        self.test_summary_frame.grid(row=0, column=0)

        self.login_frame = LoginScene(self, self.master_frame, self.data_holder)
        self.login_frame.grid(row=0, column=0)
        
        self.visual_frame = Inspection1(self, self.master_frame, self.data_holder)
        self.visual_frame.grid(row=0, column=0)
 
        self.scan_frame = ScanScene(self, self.master_frame, self.data_holder)
        self.scan_frame.grid(row=0, column=0)

        self.test1_frame= Test1Scene(self, self.master_frame, self.data_holder, 
                            "General Resistance Test",
                            queue
                            )
        self.test1_frame.grid(row=0, column=0)

        self.test2_frame= Test2Scene(self, self.master_frame, self.data_holder,
                            "ID Resistor Test", 
                            queue
                            )
        self.test2_frame.grid(row=0, column=0)

        self.test3_frame= Test3Scene(self, self.master_frame, self.data_holder, 
                            "I2C Comm. Test", 
                            queue
                            )
        self.test3_frame.grid(row=0, column=0)

        self.test4_frame= Test4Scene(self, self.master_frame, self.data_holder, 
                            "Bit Rate Test", 
                            queue
                            )
        self.test4_frame.grid(row=0, column=0)

        self.test_in_progress_frame = TestInProgressScene(self, self.master_frame, self.data_holder, queue, conn)
        self.test_in_progress_frame.grid(row=0, column=0)

        
        self.add_user_frame = AddUserScene(self, self.master_frame, self.data_holder)
        self.add_user_frame.grid(row=0, column=0)

        # Near bottom so it can reference other frames with its code
        self.splash_frame = SplashScene(self, self.master_frame)
        self.splash_frame.grid(row=0, column=0)

        #################################################
        #              End Frame Creation               #
        #################################################
        
        logging.info("GUIWindow: All frames have been created.")


        # Tells the master window that its exit window button is being given a new function
        self.master_window.protocol('WM_DELETE_WINDOW', self.exit_function)
        
        # Sets the current frame to the splash frame
        self.set_frame_splash_frame()
        self.master_frame.update() 
        self.master_frame.after(50, self.set_frame_login_frame)

        self.master_window.mainloop()
        

    #################################################

    def set_frame_add_user_frame(self):
        self.add_user_frame.update_frame(self)
        self.set_frame(self.add_user_frame)
        
        logging.debug("GUIWindow: The frame has been set to add_user_frame.")

    #################################################

    def set_frame_login_frame(self):  

        self.login_frame.update_frame(self)
        self.set_frame(self.login_frame)        

        logging.debug("GUIWindow: The frame has been set to login_frame.")
    #################################################

    def set_frame_scan_frame(self):

        self.scan_frame.is_current_scene = True
        self.set_frame(self.scan_frame)
        self.scan_frame.scan_QR_code(self.master_window)
        
        logging.debug("GUIWindow: The frame has been set to scan_frame.")

    #################################################

    def set_frame_splash_frame(self):

        self.set_frame(self.splash_frame)

        # Disables all buttons when the splash frame is the only frame
        self.sidebar.disable_all_btns()

        logging.debug("GUIWindow: The frame has been set to splash_frame.")

    #################################################

    def set_frame_visual_frame(self):
        self.visual_frame.update_frame(self)
        self.set_frame(self.visual_frame)

        logging.debug("GUIWindow: The frame has been set to visual_frame.")


    #################################################

    def scan_frame_progress(self):
        if self.data_holder.data_dict['is_new_board'] == True:
            self.set_frame_visual_frame()
        elif self.data_holder.data_dict['is_new_board'] == False:
            self.go_to_next_test()


    #################################################

    # For example, when we set the frame to test2_frame, we want to send the results
    # of test1 because it just completed.



    def set_frame_test_summary(self):
        self.test_summary_frame.update_frame()
        self.check_if_test_passed()
        self.set_frame(self.test_summary_frame)
        

        logging.debug("GUIWindow: The frame has been set to test_summary_frame.")

    #################################################

    def set_frame_test1(self):
        self.test1_frame.update_frame(self)
        self.set_frame(self.test1_frame)

        logging.debug("GUIWindow: The frame has been set to test1_frame.")
    #################################################

    def set_frame_test2(self):
        self.test2_frame.update_frame(self)
        self.set_frame(self.test2_frame)


        logging.debug("GUIWindow: The frame has been set to test2_frame.")
    #################################################

    def set_frame_test3(self):
        self.test3_frame.update_frame(self)
        self.set_frame(self.test3_frame)
        

        logging.debug("GUIWindow: The frame has been set to test3_frame.")
    #################################################

    def set_frame_test4(self):
        self.test4_frame.update_frame(self)
        self.set_frame(self.test4_frame)


        logging.debug("GUIWindow: The frame has been set to test4_frame.")
    #################################################

    def set_frame_test_in_progress(self, queue):
        self.set_frame(self.test_in_progress_frame)
        self.sidebar.disable_all_btns()
        self.test_in_progress_frame.begin_update(self.master_window, queue)
        self.go_to_next_test()   

        logging.debug("GUIWindow: The frame has been set to test_in_progress_frame.")
    #################################################

    def check_if_test_passed(self):
        logging.debug("GUIWindow: The method check_if_test_passed(self) has been called. This method is empty.")
    #################################################

    def go_to_next_test(self):
        
        # Array of potentially uncompleted tests
        test_completed_list = self.data_holder.data_lists['test_completion']
        

        test_incomplete = False
        

        logging.info("GUIWindow: Testing which tests have been completed.")
        # Checks tells the function which frame to set based on what frame is currently up
        for index, test in enumerate(test_completed_list):
            
            if test == True and index >=  self.current_test_index:
                print("Test", index + 1, "== True")
                if self.completed_window_alive == False:
                    self.completed_window_popup()
                else:
                    pass
            elif test == True:
                pass
            else:
                test_incomplete = True
                if (index ==0):
                    self.set_frame_test1()
                    self.current_test_index = 1
                elif (index == 1):
                    self.set_frame_test2()
                    self.current_test_index = 2
                elif (index == 2):
                    self.set_frame_test3()
                    self.current_test_index = 3
                elif (index == 3):
                    self.set_frame_test4()
                    self.current_test_index = 4
                break
        


        # Tests if all the tests have been completed
        # if true, brings user to Test Summary Frame rather than the next test
        if (not test_incomplete):
            self.set_frame_test_summary()

    def reset_board(self):
        self.current_test_index = 0
        self.set_frame_scan_frame()

    #################################################

    # Called to change the frame to the argument _frame
    def set_frame(self, _frame):
        
        # Updates the sidebar every time the frame is set
        self.sidebar.update_sidebar(self)

        # If frame is test_in_progress frame, disable the close program button
        # Tells the master window that its exit window button is being given a new function
        if _frame is self.test_in_progress_frame:
            self.master_window.protocol('WM_DELETE_WINDOW', self.unable_to_exit)
        else:
            # Tells the master window that its exit window button is being given a new function
            self.master_window.protocol('WM_DELETE_WINDOW', self.exit_function)
 
        #############################################################################
        #  The Following Code Determines What Buttons Are Visible On The Side Bar   #
        #############################################################################

        # Disables all but login button when on login_frame
        if _frame is self.login_frame:
            self.sidebar.disable_all_btns_but_login()

        # Disables all but scan button when on scan_frame
        if _frame is self.scan_frame:
            self.sidebar.disable_all_btns_but_scan()

        # Disables the sidebar login button when the login frame is not the current frame
        # or when scan_frame is not the current frame
        if (_frame is not self.login_frame):
            self.sidebar.disable_login_button()
            

        # Hides the submit button on scan frame until an entry is given to the computer
        if (_frame is not self.scan_frame):
            self.scan_frame.is_current_scene = False
            self.scan_frame.hide_submit_button()
            
            # Disables the sidebar scan button when the scan frame is not the current frame
            self.sidebar.disable_scan_button()

        #############################################################################
        #                        End Button Visibility Code                         #
        #############################################################################
        
        logging.debug("GUIWindow: Sidebar buttons have been updated.")

        # Brings up the test_failed popup if the test is false, continues on if not
        # Also tests the current test index so that in the event you are retrying a test it will not prompt 
        # the user about the previous test failing
        if _frame == self.test2_frame and self.current_test_index == 1:
           if self.data_holder.data_dict['test1_pass'] == False:
                TestFailedPopup(self, self.test1_frame, self.data_holder)
        if _frame == self.test3_frame and self.current_test_index == 2:
            if self.data_holder.data_dict['test2_pass'] == False:
                TestFailedPopup(self, self.test2_frame, self.data_holder)
        if _frame == self.test4_frame and self.current_test_index == 3:
            if self.data_holder.data_dict['test3_pass'] == False:
                TestFailedPopup(self, self.test3_frame, self.data_holder)
        if _frame == self.test_summary_frame and self.current_test_index == 4:
            if self.data_holder.data_dict['test4_pass'] == False:
                TestFailedPopup(self, self.test4_frame, self.data_holder)

        # Raises the passed in frame to be the current frame
        _frame.tkraise()

        logging.info("GUIWindow: The frame has been raised.")

    #################################################


    def unable_to_exit(self):
        
        logging.debug("GUIWindow: The user tried to exit during a test in progress.")

        # Creates a popup to confirm whether or not to exit out of the window
        self.popup = tk.Toplevel()
        # popup.wm_attributes('-toolwindow', 'True')
        self.popup.title("Exit Window") 
        self.popup.geometry("300x150+500+300")
        self.popup.grab_set()
       

        # Creates frame in the new window
        frm_popup = tk.Frame(self.popup)
        frm_popup.pack()

        # Creates label in the frame
        lbl_popup = tk.Label(
            frm_popup, 
            text = " You cannot exit the program \n during a test! ",
            font = ('Arial', 13)
            )
        lbl_popup.grid(column = 0, row = 0, columnspan = 2, pady = 25)


        btn_ok = tk.Button(
            frm_popup,
            width = 12,
            height = 2,
            text = "OK",
            font = ('Arial', 12),
            relief = tk.RAISED,
            command = lambda: self.destroy_popup()
        )
        btn_ok.grid(column = 0, row = 1, columnspan=2)

    #################################################
    
    # Called when a test is skipped because it has been previously passed
    def completed_window_popup(self):
        
        self.completed_window_alive = True
       
        # Creates a popup to inform user about the passing of a test
        self.popup = tk.Toplevel()
        # popup.wm_attributes('-toolwindow', 'True')
        self.popup.title("Information Window") 
        self.popup.geometry("300x150+500+300")
        self.popup.grab_set()
       

        # Creates frame in the new window
        frm_popup = tk.Frame(self.popup)
        frm_popup.pack()

        # Creates label in the frame
        lbl_popup = tk.Label(
            frm_popup, 
            text = "A test has been skipped because it\n has been previously passed.",
            font = ('Arial', 13)
            )
        lbl_popup.grid(column = 0, row = 0, pady = 25)

        # Creates yes and no buttons for exiting
        btn_okay = tk.Button(
            frm_popup,     
            width = 12,
            height = 2,
            text = "OK", 
            relief = tk.RAISED,
            font = ('Arial', 12), 
            command = lambda: self.destroy_popup()
            )
        btn_okay.grid(column = 0, row = 1)
    # Called when the no button is pressed to destroy popup and return you to the main window
    def destroy_popup(self):
        try:
            self.popup.destroy()
            self.completed_window_alive = False
            logging.debug("GUIWindow: The popup has been destroyed.")
        except:
            logging.error("GUIWindow: The popup has not been destroyed.")
    # New function for clicking on the exit button
    def exit_function(self):

        # Creates a popup to confirm whether or not to exit out of the window
        self.popup = tk.Toplevel()
        # popup.wm_attributes('-toolwindow', 'True')
        self.popup.title("Exit Window") 
        self.popup.geometry("300x150+500+300")
        self.popup.grab_set()
       

        # Creates frame in the new window
        frm_popup = tk.Frame(self.popup)
        frm_popup.pack()

        # Creates label in the frame
        lbl_popup = tk.Label(
            frm_popup, 
            text = "Are you sure you would like to exit?",
            font = ('Arial', 13)
            )
        lbl_popup.grid(column = 0, row = 0, columnspan = 2, pady = 25)

        # Creates yes and no buttons for exiting
        btn_yes = tk.Button(
            frm_popup,     
            width = 12,
            height = 2,
            text = "Yes", 
            relief = tk.RAISED,
            font = ('Arial', 12), 
            command = lambda: self.destroy_function()
            )
        btn_yes.grid(column = 0, row = 1)

        btn_no = tk.Button(
            frm_popup,
            width = 12,
            height = 2,
            text = "No",
            relief = tk.RAISED,
            font = ('Arial', 12),
            command = lambda: self.destroy_popup()
        )
        btn_no.grid(column = 1, row = 1)
        


    #################################################


    # Called when the yes button is pressed to destroy both windows
    def destroy_function(self):
        try:
            logging.info("GUIWindow: Exiting the GUI.")

            self.master_window.update()
            self.popup.update()

            if self.scan_frame.is_current_scene == True:
                self.test_in_progress_frame.close_prgbar()
                self.scan_frame.kill_processes()

            # Destroys the popup and master window
            self.popup.destroy()
            self.popup.quit()

            self.master_window.destroy()
            self.master_window.quit()

            logging.info("GUIWindow: The application has exited successfully.")
        except Exception as e:
            logging.debug("GUIWindow: " + e)
            logging.error("GUIWindow: The application has failed to close.")
            if self.retry_attempt == False:    
                logging.info("GUIWindow: Retrying...")
                self.destroy_function()
                self.retry_attempt = True


    #################################################

    
#################################################################################



#################################################################################

# importing necessary modules
import tkinter as tk
import logging
import WagonTestGUI

#################################################################################

FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'
logging.basicConfig(filename="{}/PythonFiles/logs/GUIWindow.log".format(WagonTestGUI.__path__[0]), filemode = 'w', format=FORMAT, level=logging.DEBUG)


# Creates a class that is called by the GUIWindow. 
# GUIWindow instantiates an object called login_frame.
# @param parent -> passes in GUIWindow as the parent.
# @param master_frame -> passes master_frame as the container for everything in the class.
# @param data_holder -> passes data_holder into the class so the data_holder functions can
#       be accessed within the class.
class LoginScene(tk.Frame):

    #################################################

    def __init__(self, parent, master_frame, data_holder):

        super().__init__(master_frame, width=850, height=500)
        self.data_holder = data_holder
        self.update_frame(parent)


    def update_frame(self, parent):

        for widget in self.winfo_children():
            widget.destroy()


        logging.info("LoginScene: Frame has been created.")


        # Creating a list of users for dropdown menu
        # Eventually need to add a way for a database to have control over this list
        User_List = self.data_holder.get_all_users()

        # Creating the title for the window
        lbl_title = tk.Label(
            self, 
            text="Please Select Your Name", 
            font=('Arial', '24')
            )
        lbl_title.pack(pady=75)

        # Creating intial value in dropdown menu
        self.user_selected = tk.StringVar(self)
        self.user_selected.set("") # default value is empty

        # Creating the dropdown menu itself
        self.opt_user_dropdown = tk.OptionMenu(
            self, 
            self.user_selected, # Tells option menu to use the created initial value
            *User_List # Tells the dropdown menu to use every index in the User_List list
            ) 
        self.opt_user_dropdown.pack(pady=15)
        self.opt_user_dropdown.config(width = 20, font = ('Arial', 13))
        self.opt_user_dropdown['menu'].configure(font = ('Arial', 12))

        # Traces when the user selects an option in the dropdown menu
        # When an option is selected, it calls the show_submit_button function
        self.user_selected.trace(
            'w', 
            lambda *args: self.show_submit_button()
            )

        # Creating the submit button
        # It does not get enabled until the user selects an option menu option
        self.btn_submit = tk.Button(
            self, 
            text="Submit",
            padx = 50,
            pady = 10, 
            relief=tk.RAISED, 
            command= lambda:  self.btn_submit_action(parent)
            )
        self.btn_submit.pack()
        self.btn_submit.config(state = 'disabled')


        # Creating the add user button
        self.btn_add_user = tk.Button(
            self, 
            text="Add User",
            padx = 20,
            pady = 5, 
            relief=tk.RAISED, 
            command= lambda:  self.btn_add_user_action(parent)
            )
        self.btn_add_user.pack(pady=70)

        # Forces frame to stay the size of the main_window
        # rather than adjusting to the size of the widgets
        self.pack_propagate(0)

    




    #################################################

    # Creates the function for the submit button command
    # @params "_parent" is also a parent like "parent", but it is a different "parent",
    # passes in GUIWindow
    def btn_submit_action(self, _parent):
            # Sets the user_ID in the data_holder to the selected user
        self.data_holder.set_user_ID(self.user_selected.get())
        # Changes frame to scan_frame
        _parent.set_frame_scan_frame()


        self.data_holder.print()

    #################################################

    # To be given commands later, for now it is a dummy function
    def btn_add_user_action(self, _parent):
        _parent.set_frame_add_user_frame()
    
    #################################################

    # A function to pack the submit button
    def show_submit_button(self):
        logging.info("LoginScene: User has been selected.")
        self.btn_submit.config(state = 'active')
    
    #################################################

    
#################################################################################



# Importing necessary modules
import zmq, logging
import WagonTestGUI

FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'
logging.basicConfig(filename="{}/PythonFiles/logs/SUBClient.log".format(WagonTestGUI.__path__[0]), filemode = 'w', format=FORMAT, level=logging.INFO)


# Creating a class for the SUBSCRIBE socket-type Client
class SUBClient():

    def __init__(self, conn, queue):
        with open("{}/PythonFiles/utils/server_ip.txt".format(WagonTestGUI.__path__[0]), "r") as openfile:
            grabbed_ip = openfile.read()
        logging.info("SUBClient has started") 
        # Insantiates variables       
        self.conn = conn
        self.message = ""
        # Creates the zmq.Context object
        cxt = zmq.Context()
        # Creates the socket as the SUBSCRIBE type
        listen_socket = cxt.socket(zmq.SUB)
        listen_socket.connect("tcp://{ip_address}:5556".format(ip_address = grabbed_ip))
        # Sets the topics that the server will listen for
        listen_socket.setsockopt(zmq.SUBSCRIBE, b'print')
        listen_socket.setsockopt(zmq.SUBSCRIBE, b'JSON')

        try:
            while 1 > 0:
                # Splits up every message that is received into topic and message
                # the space around the semi-colon is necessary otherwise the topic and messaage
                # will have extra spaces.
                self.topic, self.message = listen_socket.recv_string().split(" ; ")
                logging.debug("The received topic is: %s" % self.topic)
                logging.debug("The received message is: %s" % self.message)

                # Tests what topic was received and then does the appropriate code accordingly
                if self.topic == "print":

                    # Places the message in the queue. the queue.get() is in 
                    # TestInProgressScene's begin_update() method
                    queue.put(self.message)
                    logging.debug("SUBClient: The print message has been placed into the queue.")

                elif self.topic == "JSON":
                    

                    # Places the message in the queue. the queue.get() is in 
                    # TestInProgressScene's begin_update() method
                    queue.put("Results received successfully.")
                    logging.info("SUBClient: Informed the user that the results have been received.")

                    # Sends the JSON to GUIWindow on the pipe.
                    self.conn.send(self.message)
                    logging.info("SUBClient: The JSON has been sent to the GUIWindow along the pipe.")



                else:
                    logging.error("SUBClient: Invalid topic sent. Must be 'print' or 'JSON'.")

                    # Places the message in the queue. the queue.get() is in 
                    # TestInProgressScene's begin_update() method
                    queue.put("SUBClient: An error has occurred. Check logs for more details.")

        except Exception as e:
            logging.debug(e)
            logging.critical("SUBClient: SUBClient has crashed. Please restart the software.")




#################################################################################

# importing necessary modules
import tkinter as tk
import logging
import WagonTestGUI

#################################################################################

FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'
logging.basicConfig(filename="{}/PythonFiles/logs/GUIWindow.log".format(WagonTestGUI.__path__[0]), filemode = 'w', format=FORMAT, level=logging.DEBUG)


# Creates a class that is called by the GUIWindow. 
# GUIWindow instantiates an object called add_user_scene.
# @param parent -> passes in GUIWindow as the parent.
# @param master_frame -> passes master_frame as the container for everything in the class.
# @param data_holder -> passes data_holder into the class so the data_holder functions can
#       be accessed within the class.

class AddUserScene(tk.Frame):

    #################################################

    def __init__(self, parent, master_frame, data_holder):
        super().__init__(master_frame, width=850, height=500)
        logging.info("AddUserScene: Frame has been created.")
        self.data_holder = data_holder
        self.update_frame(parent)

    def update_frame(self, parent):
        
        for widget in self.winfo_children():
            widget.destroy()
        
        # Creating the title for the window
        lbl_title = tk.Label(
            self, 
            text="Add User", 
            font=('Arial', '24')
            )
        lbl_title.pack(pady=(50,0))

        # Creating entry box for new user's name
        self.new_user_name = ""
        self.user_entry = tk.Entry(
            self,
            textvariable= self.new_user_name,
            font=('Arial', '15')
            )
        self.user_entry.pack(pady=30)

        # Creating the title for the window
        password_label = tk.Label(
            self, 
            text="Enter Admin Password", 
            font=('Arial', '20')
            )
        password_label.pack(pady=(10,0))

        # Creating entry box for new user's name
        self.password = ""
        self.user_password = tk.Entry(
            self,
            textvariable= self.password,
            font=('Arial', '15'),
            show = "*"
            )
        self.user_password.pack(pady=30)

        # Creating the submit button
        self.btn_submit = tk.Button(
            self, 
            text="Submit",
            padx = 50,
            pady = 10, 
            relief=tk.RAISED, 
            command= lambda:  self.btn_submit_action(parent)
            )
        self.btn_submit.pack()
        
        # Creating the cancel button
        self.btn_submit = tk.Button(
            self, 
            text="Cancel",
            padx = 50,
            pady = 10, 
            relief=tk.RAISED, 
            command= lambda:  self.btn_cancel_action(parent)
            )
        self.btn_submit.pack()

        # Forces frame to stay the size of the main_window
        # rather than adjusting to the size of the widgets
        self.pack_propagate(0)

    #################################################

    # Creates the function for the submit button command
    # @params "_parent" is also a parent like "parent", but it is a different "parent",
    # passes in GUIWindow
    def btn_submit_action(self, _parent):
       
        self.new_user_name = self.user_entry.get()        
        self.password = self.user_password.get() 

        # Popup to confirm that a new user is added into the DB
        cnfm_pop = ConfirmPopup(_parent, self.data_holder, self.new_user_name, self.password)


    #################################################
    
    def btn_cancel_action(self, _parent):
        
        _parent.set_frame_login_frame()

    #################################################

#################################################################################


class ConfirmPopup():
    
    #################################################

    def __init__(self, parent, data_holder, new_user_name, password):
        print("\n\n\n\n\n{}\n\n\n\n".format(parent))
        self.confirm_popup(data_holder, new_user_name, password)
        self.parent = parent    

    #################################################

    # Function to make retry or continue window if the test fails
    def confirm_popup(self, data_holder, new_user_name, password):
        self.data_holder = data_holder
        self.new_user_name = new_user_name
        self.password = password
        logging.info("ConfirmPopup: Confirming that the user wants to add {}".format(self.new_user_name))
        # Creates a popup to ask whether or not to retry the test
        self.popup = tk.Toplevel()
        self.popup.title("New User Name") 
        self.popup.geometry("300x150+500+300")
        self.popup.grab_set()

        # Creates frame in the new window
        frm_popup = tk.Frame(self.popup)
        frm_popup.pack()

        # Creates label in the frame
        lbl_popup = tk.Label(
            frm_popup, 
            text = " You are about to add {} as a user \n Are you sure? ".format(self.new_user_name),
            font = ('Arial', 13)
            )
        lbl_popup.grid(column = 0, row = 0, columnspan = 2, pady = 25)

        # Creates retry and continue buttons
        btn_retry = tk.Button(
             frm_popup,
             width = 8,
             height = 2,
             text = "Cancel", 
             relief = tk.RAISED,
             font = ('Arial', 12),
             command = lambda: self.cancel_function()
             )
        btn_retry.grid(column = 0, row = 1)

        btn_continue = tk.Button(
            frm_popup,
            width = 8,
            height = 2,
            text = "Confirm",
            relief = tk.RAISED,
            font = ('Arial', 12),
            command = lambda: self.continue_function(self.parent)
        )
        btn_continue.grid(column = 1, row = 1)


    #################################################
    
    # Called when the "cancel" button is selected
    def cancel_function(self):
        self.popup.destroy()
        
    #################################################

    # Called to continue on in the testing procedure
    def continue_function(self, _parent):
        self.popup.destroy()
            
        # Adding a new user name to data_holder/DB 
        logging.info("AddUserScene: Going to DataHolder to try to add {} as a username".format(self.new_user_name))
        self.data_holder.add_new_user_name(self.new_user_name, self.password)
        # Changes frame to scan_frame
        _parent.set_frame_login_frame()

        self.data_holder.print()



#################################################################################



#################################################################################

import tkinter as tk
import logging
import WagonTestGUI
    
#################################################################################

FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'
logging.basicConfig(filename="{}/WagonTestGUI/PythonFiles/logs/GUIWindow.log".format(WagonTestGUI.__path__[0]), filemode = 'w', format=FORMAT, level=logging.DEBUG)


class TestFailedPopup():
    
    #################################################

    def __init__(self, parent, previous_frame, data_holder):
        self.test_failed_popup(parent, previous_frame, data_holder)
    
    #################################################

    # Function to make retry or continue window if the test fails
    def test_failed_popup(self, parent, previous_frame, data_holder):
        self.data_holder = data_holder
        logging.info("TestFailedPopup: A test has been failed. Prompting user for 'retry' or 'continue'.")
        # Creates a popup to ask whether or not to retry the test
        self.popup = tk.Toplevel()
        self.popup.title("Test Failed") 
        self.popup.geometry("300x150+500+300")
        self.popup.grab_set()

        # Creates frame in the new window
        frm_popup = tk.Frame(self.popup)
        frm_popup.pack()

        # Creates label in the frame
        lbl_popup = tk.Label(
            frm_popup, 
            text = " The board failed the test. \n Would you like to retry? ",
            font = ('Arial', 13)
            )
        lbl_popup.grid(column = 0, row = 0, columnspan = 2, pady = 25)

        # Creates retry and continue buttons
        btn_retry = tk.Button(
             frm_popup,
             width = 8,
             height = 2,
             text = "Retry", 
             relief = tk.RAISED,
             font = ('Arial', 12),
             command = lambda: self.retry_function(parent, previous_frame)
             )
        btn_retry.grid(column = 0, row = 1)

        btn_continue = tk.Button(
            frm_popup,
            width = 8,
            height = 2,
            text = "Continue",
            relief = tk.RAISED,
            font = ('Arial', 12),
            command = lambda: self.continue_function()
        )
        btn_continue.grid(column = 1, row = 1)

    #################################################
    
    # Called when the no button is pressed to destroy popup and return you to the main window
    def retry_function(self, parent, previous_frame):
        self.popup.destroy()
        parent.set_frame(previous_frame)
        
    #################################################

    # Called to continue on in the testing procedure
    def continue_function(self):
        self.popup.destroy()

    #################################################


#################################################################################



import logging, time
import WagonTestGUI
from PythonFiles.utils.REQClient import REQClient

# Format for the logging configuration
FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'

# Configures the logging. Set level to 'logging.INFO' to see relevant information. Set level to 'logging.DEBUG' if you wish to see every received message (only run this for a couple minutes on DEBUG, the file size is immmense.
logging.basicConfig(
    filename="{}/PythonFiles/logs/StressTest.log".format(WagonTestGUI.__path__[0]), 
    filemode = 'w', 
    format=FORMAT, 
    level=logging.INFO
    )

class StressTest():
    def __init__(self, conn, queue):
        
        #### Test Settings #### 

        # Reduce the desired time to have no remainders
        # Example:
        #           Bad ->      max_hr = 36
        #           Good ->     max_day = 1
        #                       max_hr = 12

        max_day = 0     # Set equal to to the number of desired days for the test to run       
        max_hr = 12      # Set equal to the number of desired hours for the test to run
        max_min = 0     # Set equal to the number of desired minutes for the test to run
        max_sec = 0     # Set equal to the number of desired seconds for the test to run


        #######################
        
        # Creates initial variables
        test_active = False         # Used to make sure test requests are not sent during an active test
        run_num = 0                 # Establishes the Number of Runs for information reporting
        start_time = time.time()    # Establishes a start time to report length of tests


        try:
            # Constantly attempts to start a run so long as there is not a currently active run
            # or as long as the test end requirements haven't been met
            while 1 > 0:
                # Creates a run so long as there is not currently a run happening
                if test_active == False:
                    run_start_time = time.time()                                # Logs the run start time
                    req_client = REQClient("STRESS", "000000", "STRESS")        # Starts the run by sending the test request
                    run_num += 1                                                # Increases the Run Number so that it can be tracked
                    test_active = True                                          # Makes it so that another run cannot be initialized 
                    # Is necessary for the run, is constantly trying to print the queue objects
                    # for more info, view update_console in TestInProgressScene.py
                    while 1 > 0:
                        try:
                            if not queue.empty():    
                                logging.debug("StressTest: Waiting for queue objects...")
                                text = queue.get()
                                logging.debug("Message: %s" % text)
                                if text == "Results received successfully.":
                                    # On the end of the run, logs the run information and sets the conditions for starting a new run
                                    run_time = self.determine_time(run_start_time, "string")
                                    print("StressTest: Run %s completed with a run time of %s. Beginning next run..." % (str(run_num), run_time))
                                    logging.info("StressTest: Run %s completed with a run time of %s. Beginning next run..." % (str(run_num), run_time))
                                    test_active = False
                                    break                
                            else:
                                time.sleep(.01)
                        except Exception as e:
                            # Records the error and run information
                            run_time = self.determine_time(run_start_time, "string")
                            logging.error(e)
                            logging.error("StressTest: An error has occurred during Run %s inside the nested while loop. Run time was %s" % (str(run_num), run_time))
                    # Grabs the test running time for the conditional
                    run_day, run_hr, run_min, run_sec = self.determine_time(start_time, "integer")
                    # Here is where you set the conditional to determine how many runs you want it to run for.
                    if run_day >= max_day and run_hr >= max_hr and run_min >= max_min and run_sec >= max_sec:
                        break                                             
                else:
                    pass
            # Assuming everyting went well, this will send a test completion message with information
            test_time = self.determine_time(start_time, "string")
            logging.info("StressTest: Test completed. The test time was %s and contained %s runs." % (test_time, str(run_num)))
        except Exception as e:
            # Records the error and run information
            test_time = self.determine_time(start_time, "string")
            logging.error(e)
            logging.error("StressTest: An error has occurred in the outer while loop during Run %s. The error occurred after %s" % (str(run_num), test_time))

    # Takes the a start time and the type of data you would like returned 
    # Calculates the amount of time since that start time
    # Returns that time in reduced day, hour, minute, second format.
    def determine_time(self, start_time, return_type):
        run_day = 0
        run_hr = 0
        run_min = 0
        run_sec = 0
        run_time = time.time() - start_time
        working_run_time = run_time
        if working_run_time > 60:
            run_min = int(working_run_time / 60)
            run_sec = working_run_time % 60
            working_run_time = run_min
            if working_run_time > 60:
                run_hr = int(working_run_time / 60) 
                run_min = working_run_time % 60
                working_run_time = run_hr
                if working_run_time > 24:
                    run_day = (working_run_time / 24)
                    run_hr = working_run_time % 24
        results = "%s days, %s, hours, %s minutes, and %s seconds" % (run_day, run_hr, run_min, run_sec)
        if return_type == "string":
            return results
        elif return_type == "integer":
            return run_day, run_hr, run_min, run_sec



import subprocess
import time
import signal
import ctypes
import WagonTestGUI
libc = ctypes.CDLL("libc.so.6")

from multiprocessing import Process, Manager, Pipe

def decode(hex_str):
    serial = ""
    for h in hex_str.split(" "):
        serial += bytes.fromhex(h).decode("ASCII")

    return serial

def parse_xml(inXML):
    if "<" not in inXML:
        return
    else:
        splitting = inXML.split("datalabel")
        return decode(splitting[1][1:-2])

def set_pdeathsig(sig = signal.SIGTERM):
    def callable():
        return libc.prctl(1, sig)
    return callable

def scan():
    proc = subprocess.Popen('{}/PythonFiles/Scanner/bin/runScanner'.format(WagonTestGUI.__path__[0]), stdout=subprocess.PIPE, preexec_fn=set_pdeathsig(signal.SIGTERM))
    print("Starting scanner")
    return proc
    #for line in proc.stdout:
    #    if line is not None:
    #        conn.send(line.strip().decode('utf-8'))
    #        return

def listen(serial, proc):
    for line in proc.stdout:
        if line is not None:
            print(line.strip().decode('utf-8'))
            serial.append(line.strip().decode('utf-8'))
            return

    #while not output_found:
    #    output = conn.recv()
    #    if output is not None:
    #        serial.append(output)
    #        output_found = True
    #    else:
    #        print('Still waiting')

def run_scanner():
    manager = Manager()
    serial = manager.list()

    proc = scan()
    listener = Process(target=listen, args=(serial, proc))

    listener.start()

    listener.join()

    print(parse_xml(serial[0]))

if __name__=="__main__":
    run_scanner()



#################################################################################

import tkinter as tk
from PIL import ImageTk as iTK
from PIL import Image
import logging
import WagonTestGUI

#################################################################################

FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'
logging.basicConfig(filename="{}/PythonFiles/logs/GUIWindow.log".format(WagonTestGUI.__path__[0]), filemode = 'w', format=FORMAT, level=logging.DEBUG)

class SidebarScene(tk.Frame):

    #################################################

    def __init__(self, parent, sidebar_frame, data_holder):
        super().__init__(sidebar_frame, width=213, height = 500, bg = '#808080', padx = 10, pady=10)

        self.data_holder = data_holder

        self.update_sidebar(parent)

    #################################################

    def update_sidebar(self, parent):
        
        logging.info("SidebarScene: The sidebar has been updated.")

        # List for creating check marks with for loop
        self.list_of_pass_fail = self.data_holder.data_lists['test_results']

        # For loop to create checkmarks based on pass/fail
        for index in range(len(self.list_of_pass_fail)):
            if(self.list_of_pass_fail[index] == True):
                # Create a photoimage object of the QR Code
                Green_Check_Image = Image.open("{}/PythonFiles/Images/GreenCheckMark.png".format(WagonTestGUI.__path__[0]))
                Green_Check_Image = Green_Check_Image.resize((50,50), Image.ANTIALIAS)
                Green_Check_PhotoImage = iTK.PhotoImage(Green_Check_Image)
                GreenCheck_Label = tk.Label(self, image=Green_Check_PhotoImage, width=50, height=50, bg = '#808080')
                GreenCheck_Label.image = Green_Check_PhotoImage

                GreenCheck_Label.grid(row=index + 2, column=1)

            else:
                # Create a photoimage object of the QR Code
                Red_X_Image = Image.open("{}/PythonFiles/Images/RedX.png".format(WagonTestGUI.__path__[0]))
                Red_X_Image = Red_X_Image.resize((50,50), Image.ANTIALIAS)
                Red_X_PhotoImage = iTK.PhotoImage(Red_X_Image)
                RedX_Label = tk.Label(self, image=Red_X_PhotoImage, width=50, height=50, bg = '#808080')
                RedX_Label.image = Red_X_PhotoImage

                RedX_Label.grid(row=index + 2, column=1)

        # Variables for easy button editing
        btn_height = 3
        btn_width = 18
        btn_font = ('Arial', 10)
        btn_pady = 5

        self.btn_login = tk.Button(
            self,
            pady = btn_pady,
            text = 'LOGIN PAGE',
            height = btn_height,
            width = btn_width,
            font = btn_font
        )
        self.btn_login.grid(column = 0, row = 0)

        self.btn_scan = tk.Button(
            self,
            pady = btn_pady,
            text = 'SCAN PAGE',
            height = btn_height,
            width = btn_width,
            font = btn_font
        )
        self.btn_scan.grid(column = 0, row = 1)

        self.btn_test1 = tk.Button(
            self, 
            pady = btn_pady,
            text = 'GEN. RESIST. TEST',
            height = btn_height,
            width = btn_width,
            font = btn_font,
            command = lambda: self.btn_test1_action(parent)
            )
        self.btn_test1.grid(column = 0, row = 2)

        if self.data_holder.data_dict['test1_pass'] == True:
            self.btn_test1.config(state = 'disabled')


        self.btn_test2 = tk.Button(
            self,
            pady = btn_pady, 
            text = 'ID RESISTOR TEST',
            height = btn_height,
            width = btn_width,
            font = btn_font,
            command = lambda: self.btn_test2_action(parent)
            )
        self.btn_test2.grid(column = 0, row = 3)

        if self.data_holder.data_dict['test2_pass'] == True:
            self.btn_test2.config(state = 'disabled')

        self.btn_test3 = tk.Button(
            self, 
            pady = btn_pady,
            text = 'I2C COMM. TEST',
            height = btn_height,
            width = btn_width,
            font = btn_font,
            command = lambda: self.btn_test3_action(parent)
            )
        self.btn_test3.grid(column = 0, row = 4)

        if self.data_holder.data_dict['test3_pass'] == True:
            self.btn_test3.config(state = 'disabled')

        self.btn_test4 = tk.Button(
            self, 
            pady = btn_pady,
            text = 'BIT RATE TEST',
            height = btn_height,
            width = btn_width,
            font = btn_font,
            command = lambda: self.btn_test4_action(parent)
            )
        self.btn_test4.grid(column = 0, row = 5)
        if self.data_holder.data_dict['test4_pass'] == True:
            self.btn_test4.config(state = 'disabled')

        self.btn_summary = tk.Button(
            self, 
            pady = btn_pady,
            text = 'TEST SUMMARY',
            height = btn_height,
            width = btn_width,
            font = btn_font,
            command = lambda: self.btn_summary_action(parent)
            )
        self.btn_summary.grid(column = 0, row = 6)

        self.grid_propagate(0)

    #################################################

    def btn_test1_action(self, _parent):
        _parent.set_frame_test1()

    def btn_test2_action(self, _parent):
        _parent.set_frame_test2()

    def btn_test3_action(self, _parent):
        _parent.set_frame_test3()

    def btn_test4_action(self, _parent):
        _parent.set_frame_test4()

    def btn_summary_action(self, _parent):
        _parent.set_frame_test_summary()

    #################################################

    def disable_all_btns(self):
        self.btn_login.config(state = 'disabled')
        self.btn_scan.config(state = 'disabled')
        self.btn_test1.config(state = 'disabled')
        self.btn_test2.config(state = 'disabled')
        self.btn_test3.config(state = 'disabled')
        self.btn_test4.config(state = 'disabled')
        self.btn_summary.config(state = 'disabled')

    #################################################

    def disable_all_but_log_scan(self):
        self.btn_test1.config(state = 'disabled')
        self.btn_test2.config(state = 'disabled')
        self.btn_test3.config(state = 'disabled')
        self.btn_test4.config(state = 'disabled')
        self.btn_summary.config(state = 'disabled')

    #################################################

    def disable_all_btns_but_scan(self):
        self.btn_login.config(state = 'disabled')
        self.btn_test1.config(state = 'disabled')
        self.btn_test2.config(state = 'disabled')
        self.btn_test3.config(state = 'disabled')
        self.btn_test4.config(state = 'disabled')
        self.btn_summary.config(state = 'disabled')

    #################################################

    def disable_all_btns_but_login(self):
        self.btn_login.config(state = 'normal')
        self.btn_scan.config(state = 'disabled')
        self.btn_test1.config(state = 'disabled')
        self.btn_test2.config(state = 'disabled')
        self.btn_test3.config(state = 'disabled')
        self.btn_test4.config(state = 'disabled')
        self.btn_summary.config(state = 'disabled')

    #################################################

    def disable_log_scan(self):
        self.btn_login.config(state = 'disabled')
        self.btn_scan.config(state = 'disabled')

    #################################################
        
    def disable_login_button(self):
        self.btn_login.config(state = 'disabled')

    #################################################

    def disable_scan_button(self):
        self.btn_scan.config(state = 'disabled')
    
    #################################################


#################################################################################



import string    
import random 
from random import randint


class StressScript():
    def __init__(self, conn):
        # Set random test_length within a range. Meant to be close to 3-5 min.
        test_length = randint(500,1500)
        i = 0
        while i < test_length:
            # Creates random strings of 10 characters and sends them until the test length has been reached.
            S = 10  
            ran = ''.join(random.choices(string.ascii_uppercase + string.digits, k = S))
            conn.send(ran)
            i += 1
        conn.send("Done.")
        conn.send("Run completed")
        



#################################################################################

# Importing Necessary Modules
import tkinter as tk
import tkinter.font as font
import logging
import WagonTestGUI

# Importing Necessary Files
from WagonTestGUI.PythonFiles.utils.REQClient import REQClient

#################################################################################

FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'
logging.basicConfig(filename="{}/PythonFiles/logs/GUIWindow.log".format(WagonTestGUI.__path__[0]), filemode = 'w', format=FORMAT, level=logging.DEBUG)

# Creating class for the window
class TestScene(tk.Frame):

    #################################################

    def __init__(self, parent, master_frame, data_holder, test_name, queue):
        super().__init__(master_frame, width=850, height=500)
        self.queue = queue
        self.test_name = test_name
        self.data_holder = data_holder
        
        self.update_frame(parent)

    #################################################

    def update_frame(self, parent):
        logging.debug("ParentTestClass: A test frame has been updated.")
        # Creates a font to be more easily referenced later in the code
        font_scene = ('Arial', 15)

        # Create a centralized window for information
        frm_window = tk.Frame(self, width = 850, height = 500)
        frm_window.grid(column=1, row=1, padx = 223, pady = 100)

        # Create a label for the tester's name
        lbl_tester = tk.Label(
            frm_window, 
            text = "Tester: ", 
            font = font_scene
            )
        lbl_tester.pack(side = 'top')

        # Create an entry for the tester's name
        ent_tester = tk.Entry(
            frm_window, 
            font = font_scene
            )
        ent_tester.insert(0, self.data_holder.data_dict['user_ID'])
        ent_tester.pack(side = 'top')
        ent_tester.config(state = "disabled")

        # Create a label for the serial number box
        lbl_snum = tk.Label(
            frm_window, 
            text = "Serial Number: ", 
            font = font_scene
            )
        lbl_snum.pack(side = 'top')

        # Create a entry for the serial number box
        ent_snum = tk.Entry(
            frm_window, 
            font = font_scene
            )
        ent_snum.insert(0, self.data_holder.data_dict['current_serial_ID'])
        ent_snum.pack(side = 'top')
        ent_snum.config(state = "disabled")

        # Create a label for the test about to be run
        lbl_test = tk.Label(
            frm_window, 
            text = "Current Test: ", 
            font = font_scene
            )
        lbl_test.pack(side = 'top')


        # Create a entry for the test type
        self.ent_test = tk.Entry(
            frm_window, 
            font = font_scene
            )
        self.ent_test.pack(side = 'top')
        self.ent_test.insert(0, self.test_name)
        self.ent_test.config(state = "disabled")

        # Create a label for confirming test
        lbl_confirm = tk.Label(
            frm_window, 
            text = "Are you ready to begin the test?", 
            font = font_scene
            )
        lbl_confirm.pack(side = 'top')

        # Create a button for confirming test
        btn_confirm = tk.Button(
            frm_window, 
            text = "Confirm", 
            relief = tk.RAISED, 
            command = lambda:self.btn_confirm_action(parent)
            )
        btn_confirm.pack(side = 'top')
        btn_confirm['font'] = font.Font(family = 'Arial', size = 13)

        # Create frame for logout button
        frm_logout = tk.Frame(self)
        frm_logout.grid(column = 2, row = 2, sticky = 'n')

        # Create a logout button
        btn_logout = tk.Button(
            frm_logout, 
            text = "Logout", 
            relief = tk.RAISED, 
            command = lambda: self.btn_logout_action(parent))
        btn_logout.pack(anchor = 'center')

        # Create a frame for the back button
        frm_back = tk.Frame(self)
        frm_back.grid(column = 2, row = 0, sticky = 'n')

        # Create a rescan button
        btn_rescan = tk.Button(
            frm_back, 
            text = "Change Boards", 
            relief = tk.RAISED, 
            command = lambda: self.btn_rescan_action(parent))
        btn_rescan.pack(anchor = 'n')


        

        self.grid_propagate(0)
        
    #################################################

    # Rescan button takes the user back to scanning in a new board
    def btn_rescan_action(self, _parent):
        _parent.reset_board()
    
    #################################################

    # Confirm button action takes the user to the test in progress scene
    def btn_confirm_action(self, _parent):
        pass

    #################################################

    # functionality for the logout button
    def btn_logout_action(self, _parent):
        _parent.set_frame_login_frame()

    #################################################


#################################################################################


class Test1Scene(TestScene):
    
    logging.info("Test1Scene: Frame has successfully been created.")

    # Override to add specific functionality
    def btn_confirm_action(self, _parent):

        self.data_holder.print()
        super().btn_confirm_action(_parent)
        test_1_client = REQClient('test1', self.data_holder.data_dict['current_serial_ID'], self.data_holder.data_dict['user_ID'])
        _parent.set_frame_test_in_progress(self.queue)

#################################################################################


class Test2Scene(TestScene):

    logging.info("Test2Scene: Frame has successfully been created.")

    # Override to add specific functionality
    def btn_confirm_action(self, _parent):
        self.data_holder.print()
        super().btn_confirm_action(_parent)
        test_2_client = REQClient('test2', self.data_holder.data_dict['current_serial_ID'], self.data_holder.data_dict['user_ID'])
        _parent.set_frame_test_in_progress(self.queue)
        


#################################################################################


class Test3Scene(TestScene):

    logging.info("Test3Scene: Frame has successfully been created.")

    # Override to add specific functionality
    def btn_confirm_action(self, _parent):

        self.data_holder.print()
        super().btn_confirm_action(_parent)
        test_3_client = REQClient('test3', self.data_holder.data_dict['current_serial_ID'], self.data_holder.data_dict['user_ID'])
        _parent.set_frame_test_in_progress(self.queue)


#################################################################################


class Test4Scene(TestScene):

    logging.info("Test4Scene: Frame has successfully been created.")

    # Override to add specific functionality
    def btn_confirm_action(self, _parent):

        self.data_holder.print()
        super().btn_confirm_action(_parent)
        test_4_client = REQClient('test4', self.data_holder.data_dict['current_serial_ID'], self.data_holder.data_dict['user_ID'])
        _parent.set_frame_test_in_progress(self.queue)

#################################################################################




#####################################################################
#                                                                   #
#  This is the code for a client to send a request to a server to   #
#                   run specific test scripts.                      #
#                                                                   #
#####################################################################

#################################################################################

# Importing necessary modules
import zmq, logging
import WagonTestGUI

#################################################################################

FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'
logging.basicConfig(
    filename="{}/PythonFiles/logs/StressTest.log".format(WagonTestGUI.__path__[0]), 
    filemode = 'w', 
    format=FORMAT, 
    level=logging.INFO
    )

# Making the Client Server a class
class REQClient():

    #################################################

    # Ensures nothing happens on instantiantion
    def __init__(self, desired_test, serial, tester):
        with open("{}/PythonFiles/utils/server_ip.txt".format(WagonTestGUI.__path__[0]),"r") as openfile:
            grabbed_ip = openfile.read()
        self.message = ""
        self.serial = serial
        self.tester = tester
        sending_msg = desired_test + ";" + serial + ";" + tester
        # Establishing variable for use
        self.desired_test = desired_test
        # Necessary for zmqClient    
        context = zmq.Context()

        # Creates a socket to talk to the server
        # print("Connecting to the testing server...")
        socket = context.socket(zmq.REQ)
        socket.connect("tcp://{ip_address}:5555".format(ip_address = grabbed_ip))

        debug_msg = "REQClient: Sending request to REPServer for: " + self.desired_test
        logging.debug(debug_msg)
        # Tell the server what test to run
        socket.send_string(sending_msg)
        logging.debug("REQClient: Request sent. Waiting for confirmation receipt...")
        # Get the reply
        self.message = socket.recv_string()
        print("REQClient: Request received.")

        # Closes the client so the code in the GUI can continue once the request is sent.
        socket.close()

    #################################################

    def get_message(self):
        return self.message

    #################################################

    def set_message(self, string):
        self.message = string

    #################################################
    
            
#################################################################################



from setuptools import setup, find_packages
from pathlib import Path

print("\n\n\n\n\n{}\n\n\n\n".format( find_packages(where='.')))

this_directory = Path(__file__).parent


setup(
    name='WagonTestGUI',
    version='1.2.4', 
    description='Python scripts to be located on the user interface computer.',
    long_description = (this_directory / "README.md").read_text(),
    long_description_content_type='text/markdown',
    url='https://github.com/UMN-CMS/WagonTestGUI',
    author='Garrett Schindler, Andrew Kirzeder, & Bryan Crossman',
    author_email='none@gmail.com',
    packages= find_packages(
            where='.',
            ),
    include_package_data = True,
    package_data = {
        "": ["*.png", "*.json"]
    },
    install_requires=[
            'numpy',
            'matplotlib',
            'subprocess',
            'socket',
            'audioop',
            'logging',
            'json'
            ]
    

)



#################################################################################

import WagonTestGUI
import json, logging
import tkinter as tk
from PIL import ImageTk as iTK
from PIL import Image
from matplotlib.pyplot import table
from pyparsing import col
import WagonTestGUI

#################################################################################

FORMAT = '%(asctime)s|%(levelname)s|%(message)s|'
logging.basicConfig(filename="{}/PythonFiles/logs/GUIWindow.log".format(WagonTestGUI.__path__[0]), filemode = 'w', format=FORMAT, level=logging.DEBUG)

# Frame that shows all of the final test results
# @param parent -> References a GUIWindow object
# @param master_frame -> Tkinter object that the frame is going to be placed on
# @param data_holder -> DataHolder object that stores all relevant data

class TestSummaryScene(tk.Frame):

    #################################################

    def __init__(self, parent, master_frame, data_holder):
    
        self.parent = parent
        
        # Call to the super class's constructor
        # Super class is the tk.Frame class
        super().__init__(master_frame, width=850, height=500)

        logging.info("TestSummaryScene: Frame has been created.")

        self.data_holder = data_holder

        # Setting weights of columns so the column 4 is half the size of columns 0-3
        self.columnconfigure(0, weight = 2)
        self.columnconfigure(1, weight = 2)
        self.columnconfigure(2, weight = 2)
        self.columnconfigure(3, weight = 2)
        self.columnconfigure(4, weight = 1)
        # Instantiates an updated table with the current data
        self.create_updated_table(parent)

        # Adds the title to the TestSummary Frame
        self.title = tk.Label(
                self, 
                fg='#0d0d0d', 
                text = "Testing Finished!",
                font=('Arial',18,'bold')
                )
        self.title.grid(row= 0, column= 1, pady = 20)

        # Fits the frame to set size rather than interior widgets
        self.grid_propagate(0)

    #################################################

    
    # Creates the table with the updated information from the data_holder
    # @param parent -> References the GUIWindow object that creates the class
    
    def create_updated_table(self, parent):

        logging.debug("TestSummaryScene: Table is being updated.")        
        
        self.list_of_tests = ["General Resistance Test", "ID Resistor Test", "I2C Comm. Test", "Bit Rate Test"]
        self.list_of_table_labels = ["Test Name", "Test Status", "Pass/Fail"]
        self.list_of_completed_tests = self.data_holder.data_lists['test_completion']
        self.list_of_pass_fail = self.data_holder.data_lists['test_results']


        # Adds Board Serial Number to the TestSummaryFrame
        self.lbl_snum = tk.Label(
                self, 
                text = "Serial Number: " + str(self.data_holder.data_dict['current_serial_ID']),
                font=('Arial', 14)
                )
        self.lbl_snum.grid(column = 2, row = 0, pady = 20)

        # Adds Tester Name to the TestSummary Frame
        self.lbl_tester = tk.Label(
                self, 
                text = "Tester: " + self.data_holder.data_dict['user_ID'],
                font=('Arial', 14)
                )
        self.lbl_tester.grid(column = 0, row = 0, pady = 20)
       
        
        # Creates the "table" as a frame object
        self.frm_table = tk.Frame(self)
        self.frm_table.grid(row = 1, column= 0, columnspan = 4, rowspan = 4)
        
        # Setting weights of columns so the column 4 is half the size of columns 0-3
        self.frm_table.columnconfigure(0, weight = 2)
        self.frm_table.columnconfigure(1, weight = 2)
        self.frm_table.columnconfigure(2, weight = 2)
        self.frm_table.columnconfigure(3, weight = 1)
        self.frm_table.columnconfigure(4, weight = 1)
        

        
        # Adds the labels to the top of the table
        for index in range(len(self.list_of_table_labels)):
            _label = tk.Label(
                    self.frm_table, 
                    text = self.list_of_table_labels[index], 
                    relief = 'ridge', 
                    width=25, 
                    height=1, 
                    font=('Arial', 11, "bold")
                    )
            _label.grid(row= 0, column=index)
            

        # Adds the test names to the first column
        for index in range(len(self.list_of_tests)):
            _label= tk.Label(
                    self.frm_table, 
                    text = self.list_of_tests[index], 
                    relief = 'ridge', 
                    width=25, 
                    height=5, 
                    font=('Arial', 11)
                    )
            _label.grid(row=index + 1, column=0)
            


        # Create Labels that tell whether or not a test was completed
        for index in range(len(self.list_of_completed_tests)):
            
            # Instantiates a Label
            _label = tk.Label(
                        self.frm_table,
                        relief = 'ridge', 
                        width=25, 
                        height=5, 
                        font=('Arial',11)
                        )

            # if the test is completed, set the label to "Complete"
            if (self.list_of_completed_tests[index]):
                _label.config(
                        text = "COMPLETED"
                        )
            # else, set the label to "Unfinished"
            else:
                _label.config(
                        text = "UNFINISHED"
                        )

            # Puts the completed/unfinished label into the table       
            _label.grid(row=index + 1, column=1)


        # Adds the Image as to whether the test was completed or not
        for index in range(len(self.list_of_pass_fail)):
            if(self.list_of_pass_fail[index]):
                # Create a photoimage object of the QR Code
                Green_Check_Image = Image.open("{}/PythonFiles/Images/GreenCheckMark.png".format(WagonTestGUI.__path__[0]))
                Green_Check_Image = Green_Check_Image.resize((75,75), Image.ANTIALIAS)
                Green_Check_PhotoImage = iTK.PhotoImage(Green_Check_Image)
                GreenCheck_Label = tk.Label(self.frm_table, image=Green_Check_PhotoImage, width=75, height=75)
                GreenCheck_Label.image = Green_Check_PhotoImage

                GreenCheck_Label.grid(row=index + 1, column=2)

            else:
                # Create a photoimage object of the QR Code
                Red_X_Image = Image.open("{}/PythonFiles/Images/RedX.png".format(WagonTestGUI.__path__[0]))
                Red_X_Image = Red_X_Image.resize((75,75), Image.ANTIALIAS)
                Red_X_PhotoImage = iTK.PhotoImage(Red_X_Image)
                RedX_Label = tk.Label(self.frm_table, image=Red_X_PhotoImage, width=75, height=75)
                RedX_Label.image = Red_X_PhotoImage

                RedX_Label.grid(row=index + 1, column=2)


        self.create_retest_more_info_btns(parent)
       
        self.grid_propagate(0)

        logging.debug("TestSummaryScene: Table finished update.")     

    #################################################

    # Creates all of the retest button
    def create_retest_more_info_btns(self, parent):

        logging.debug("TestSummaryScene: Buttons are being created.")

        row1 = tk.Frame(self.frm_table)
        row1.grid(column = 3, row = 1)
        
        btn_retest1 = tk.Button(
                row1, 
                text = "RETEST",
                padx= 5,
                pady=5,  
                command = lambda: self.btn_retest1_action(parent)
                )
        btn_retest1.grid(column = 1, row = 0, padx=5, pady=5)

        btn_more_info1 = tk.Button(
                row1, 
                text = "MORE INFO", 
                padx= 5,
                pady=5, 
                command = lambda: self.btn_more_info1_action(parent)
                )
        btn_more_info1.grid(column=0, row = 0)




        row2 = tk.Frame(self.frm_table)
        row2.grid(column = 3, row = 2)
        
        btn_retest2 = tk.Button(
                row2, 
                text = "RETEST",
                padx= 5,
                pady=5,  
                command = lambda: self.btn_retest2_action(parent)
                )
        btn_retest2.grid(column = 1, row = 0, padx=5, pady=5)

        btn_more_info2 = tk.Button(
                row2, 
                text = "MORE INFO", 
                padx= 5,
                pady=5, 
                command = lambda: self.btn_more_info2_action(parent)
                )
        btn_more_info2.grid(column=0, row = 0)




        row3 = tk.Frame(self.frm_table)
        row3.grid(column = 3, row = 3)
        
        btn_retest3 = tk.Button(
                row3, 
                text = "RETEST",
                padx= 5,
                pady=5,  
                command = lambda: self.btn_retest3_action(parent)
                )
        btn_retest3.grid(column = 1, row = 0, padx=5, pady=5)

        btn_more_info3 = tk.Button(
                row3, 
                text = "MORE INFO", 
                padx= 5,
                pady=5, 
                command = lambda: self.btn_more_info3_action(parent)
                )
        btn_more_info3.grid(column=0, row = 0)

        
        
        
    
        
        row4 = tk.Frame(self.frm_table)
        row4.grid(column = 3, row = 4)
        
        btn_retest4 = tk.Button(
                row4, 
                text = "RETEST",
                padx= 5,
                pady=5, 
                command = lambda: self.btn_retest4_action(parent)
                )
        btn_retest4.grid(column = 1, row = 0, padx=5, pady=5)

        btn_more_info4 = tk.Button(
                row4, 
                text = "MORE INFO", 
                padx= 5,
                pady=5, 
                command = lambda: self.btn_more_info4_action(parent)
                )
        btn_more_info4.grid(column=0, row = 0)




        btn_next_test = tk.Button(
                self.frm_table, 
                text = "NEXT BOARD",
                font = ('Arial', 15), 
                command = lambda: self.btn_next_test_action(parent)
                )
        btn_next_test.grid(column = 3, row = 5)

        logging.debug("TestSummaryScene: Buttons finshed being created.")

    #################################################

    # A function to be called within GUIWindow to create the console output
    # when the frame is being brought to the top
    def create_JSON_popup(self, JSON_String):
        try:
            # Creating a popup window for the JSON Details
            self.JSON_popup = tk.Toplevel()
            self.JSON_popup.geometry("500x300+750+100")
            self.JSON_popup.title("JSON Details")
            # self.JSON_popup.wm_attributes('-toolwindow', 'True')

            self.JSON_popup.grab_set()
            self.JSON_popup.attributes('-topmost', 'true') 

            # Creating a Frame For Console Output
            frm_JSON = tk.Frame(self.JSON_popup, width = 500, height = 300, bg = 'green')
            frm_JSON.pack_propagate(0)
            frm_JSON.pack()

            # Placing an entry box in the frm_console
            self.JSON_entry_box = tk.Text(
                frm_JSON, 
                bg = '#6e5e5d', 
                fg = 'white', 
                font = ('Arial', 14)
                )
            self.JSON_entry_box.pack(anchor = 'center', fill=tk.BOTH, expand=1)

            current_JSON_file = open(JSON_String)
            current_JSON_data = json.load(current_JSON_file)


            temp = ""
            for key, value in current_JSON_data.items():
                temp = temp + "{} : {}".format(key, value) + "\n"


            self.JSON_entry_box.delete(1.0,"end")
            self.JSON_entry_box.insert(1.0, temp)
            
            current_JSON_file.close()   
        except Exception as e:
            logging.debug(e)
            logging.warning("TestSummaryScene: More Info popup has failed to be created.")

            

    #################################################

    # All of the different methods for what the retest buttons should do
    def btn_retest1_action(self, _parent):
        _parent.set_frame(_parent.test1_frame)
 
    def btn_retest2_action(self, _parent):
        _parent.set_frame(_parent.test2_frame)
        
    def btn_retest3_action(self, _parent):
        _parent.set_frame(_parent.test3_frame)

    def btn_retest4_action(self, _parent):
        _parent.set_frame(_parent.test4_frame)

    #################################################

    def btn_more_info1_action(self, _parent):
        self.create_JSON_popup("{}/PythonFiles/JSONFiles/Current_GenRes_JSON.json".format(WagonTestGUI.__path__[0]))

    def btn_more_info2_action(self, _parent):
        self.create_JSON_popup("{}/PythonFiles/JSONFiles/Current_IDRes_JSON.json".format(WagonTestGUI.__path__[0]))

    def btn_more_info3_action(self, _parent):
        self.create_JSON_popup("{}/PythonFiles/JSONFiles/Current_IIC_JSON.json".format(WagonTestGUI.__path__[0]))
    
    def btn_more_info4_action(self, _parent):
        self.create_JSON_popup("{}/PythonFiles/JSONFiles/Current_BERT_JSON.json".format(WagonTestGUI.__path__[0]))

    #################################################

    # Next test button action
    def btn_next_test_action(self, _parent):
        self.data_holder.data_holder_new_test()
        _parent.reset_board()
        logging.info("TestSummaryScene: Starting a new test.")
        
    #################################################

    # Updates the frame to show current data
    def update_frame(self):
        self.create_updated_table(self.parent)

    #################################################

    # TODO Check what this is used for
    def add_new_test(self, _list_of_completed_tests, _list_of_pass_fail):
        self.list_of_completed_tests = _list_of_completed_tests
        self.list_of_pass_fail = _list_of_pass_fail

    #################################################

#################################################################################
